package cage.carl;
import java.util.*;

class BlockNode
{
    public enum Operation
    {
        S_BLOCK,
        S_ASSIGN,
		S_ASSIGN_DOTX,
		S_ASSIGN_DOTY,
        S_IF,
        S_WHILE,
        S_RETURN,
        S_CALL,
        S_AND,
        S_OR,
        S_EQ,
        S_NE,
        S_LT,
        S_LE,
        S_GT,
        S_GE,
        S_ADD,
        S_SUB,
        S_MULT,
        S_DIV,
        S_BANG,
        S_NEG,
        S_POINT,
        S_VAL,
        S_ID,
        S_PTX,
        S_PTY
    }
    
    private final Operation op;
    private Closure.ReturnType type;
    private Object value;
    private ArrayList<BlockNode> arguments;
    
    public BlockNode(Object o)
    {
        op = Operation.S_VAL;
        
        if (o != null)
        {
            String t = o.getClass().getName();
            if (t.equals("java.lang.Integer"))
            {
                type = Closure.ReturnType.R_INT;
            }
            else if (t.equals("java.lang.Double"))
            {
                type = Closure.ReturnType.R_REAL;
            }
            else if (t.equals("java.lang.Boolean"))
            {
                type = Closure.ReturnType.R_BOOL;
            }
            else if (t.equals("java.lang.String"))
            {
                type = Closure.ReturnType.R_STR;
            }
            else if (t.equals("cage.carl.Point"))
            {
                type = Closure.ReturnType.R_PT;
            }
            else //if (t.equals("cage.carl.Closure"))
            {
                type = Closure.ReturnType.R_NOTHING;
            }
            value = o;
        }
        else
        {
            type = Closure.ReturnType.R_NOTHING;
            value = null;
        }
        
        arguments = null;
    }

    public BlockNode(Operation o, Object v, ArrayList<BlockNode> a, Closure.ReturnType t)
    {
        op = o;
        type = t;
        value = v;
        arguments = a;
    }
    
    public Operation getOp()
    {
        return op;
    }

    public Closure.ReturnType getReturnType()
    {
        return type;
    }
    
    public Object getValue()
    {
        return value;
    }
    
    public Object execute(HashMap<String,Object> globals, HashMap<String,Object> bindings)
        throws InvalidException
    {
        Boolean a;
        Boolean b;
        Object arg0;
        Object arg1;
    
        Object r = null;
    
        switch(op)
        {
            case S_BLOCK:
                // run through all the statements and execute
                // if r is not null, there was a return statement
                for (int i = 0; i < arguments.size() && r == null; i++)
                {
                    // avoid the case in which a function call returns something
                    if (arguments.get(i).getOp() != Operation.S_CALL)
                    {
                        r = arguments.get(i).execute(globals, bindings);
                    }
                    else
                    {
                        arguments.get(i).execute(globals, bindings);
                    }
                }
                break;
            case S_ASSIGN:
                // arg 0 == variable
                // arg 1 == boolop
                if (bindings.containsKey((String)arguments.get(0).getValue()))
                {
                    bindings.put((String)arguments.get(0).getValue(), arguments.get(1).execute(globals, bindings));
                }
                else
                {
                    globals.put((String)arguments.get(0).getValue(), arguments.get(1).execute(globals, bindings));
                }
                break;
			case S_ASSIGN_DOTX:
                // arg 0 == variable
                // arg 1 == boolop
                if (bindings.containsKey((String)arguments.get(0).getValue()))
                {
					Object o = arguments.get(1).execute(globals, bindings);
					if (o instanceof Integer)
					{
						Integer i = (Integer)o;
						((Point)bindings.get((String)arguments.get(0).getValue())).x = i.doubleValue();
					}
					else if (o instanceof Double)
					{
						Double d = (Double)o;
						((Point)bindings.get((String)arguments.get(0).getValue())).x = d.doubleValue();
					}
					else
					{
						throw new InvalidException("BlockNode: bad type");
					}
				}
                else
                {
					Object o = arguments.get(1).execute(globals, bindings);
					if (o instanceof Integer)
					{
						Integer i = (Integer)o;
						((Point)globals.get((String)arguments.get(0).getValue())).x = i.doubleValue();
					}
					else if (o instanceof Double)
					{
						Double d = (Double)o;
						((Point)globals.get((String)arguments.get(0).getValue())).x = d.doubleValue();
					}
					else
					{
						throw new InvalidException("BlockNode: bad type");
					}
                }
                break;
			case S_ASSIGN_DOTY:
                // arg 0 == variable
                // arg 1 == boolop
                if (bindings.containsKey((String)arguments.get(0).getValue()))
                {
					Object o = arguments.get(1).execute(globals, bindings);
					if (o instanceof Integer)
					{
						Integer i = (Integer)o;
						((Point)bindings.get((String)arguments.get(0).getValue())).y = i.doubleValue();
					}
					else if (o instanceof Double)
					{
						Double d = (Double)o;
						((Point)bindings.get((String)arguments.get(0).getValue())).y = d.doubleValue();
					}
					else
					{
						throw new InvalidException("BlockNode: bad type");
					}
				}
                else
                {
					Object o = arguments.get(1).execute(globals, bindings);
					if (o instanceof Integer)
					{
						Integer i = (Integer)o;
						((Point)globals.get((String)arguments.get(0).getValue())).y = i.doubleValue();
					}
					else if (o instanceof Double)
					{
						Double d = (Double)o;
						((Point)globals.get((String)arguments.get(0).getValue())).y = d.doubleValue();
					}
					else
					{
						throw new InvalidException("BlockNode: bad type");
					}
                }
                break;
            case S_IF:
                // arg 0 == first boolop
                // arg 1 == first block
                // arg n == elsif expr/block
                // last arg == else block
                b = (Boolean)arguments.get(0).execute(globals, bindings);
                if (b.booleanValue())
                {
                    r = arguments.get(1).execute(globals, bindings);
                }
                else if (arguments.size() > 2)
                {
                    if (arguments.size() > 3)
                    {
                        // do the elseif blocks
                        int i;
                        for (i = 2; i < arguments.size() - 1; i += 2)
                        {
                            b = (Boolean)arguments.get(i).execute(globals, bindings);
                            if (b.booleanValue())
                            {
                                r = arguments.get(i+1).execute(globals, bindings);
                                // break the loop, and don't do the else block
                                i = arguments.size();
                            }
                        }
                        // if i is still less than arguments.size(), do the else block
                        if (i < arguments.size())
                        {
                            r = arguments.get(arguments.size() - 1).execute(globals, bindings);
                        }
                    }
                    else
                    {
                        // do the else block
                        r = arguments.get(2).execute(globals, bindings);
                    }
                }
                break;
            case S_WHILE:
                // arg 0 == boolop
                // arg 1 == block
                b = (Boolean)arguments.get(0).execute(globals, bindings);
                while (b.booleanValue() && r == null)
                {
                    r = arguments.get(1).execute(globals, bindings);
                    if (r == null)
                    {
                        b = (Boolean)arguments.get(0).execute(globals, bindings);
                    }
                }
                break;
            case S_RETURN:
                // (optional) arg 0 == boolop
                if (arguments.size() > 0)
                {
                    r = arguments.get(0).execute(globals, bindings);
                }
                else
                {
                    r = new Returned();
                }
                break;
            case S_CALL:
                // arg 0 == id of function to call
                // arg n == arguments
                Closure func = (Closure)arguments.get(0).getValue();
                
                ArrayList<String> pnames = func.getParamNames();
                HashMap<String,Object> params = new HashMap<String,Object>();
                Object arg;
                
                for (int i = 1; i < arguments.size(); i++)
                {
                    arg = arguments.get(i).execute(globals, bindings);
                    params.put(pnames.get(i-1), arg);
                }
                
                r = func.execute(globals, params);
                break;
            case S_AND:
                // arg 0 == boolop
                // arg 1 == boolop
                a = (Boolean)arguments.get(0).execute(globals, bindings);
                if (a.booleanValue())
                {
                    b = (Boolean)arguments.get(1).execute(globals, bindings);
                    if (b.booleanValue())
                    {
                        r = new Boolean(true);
                    }
                    else
                    {
                        r = b;
                    }
                }
                else
                {
                    r = a;
                }
                break;
            case S_OR:
                // arg 0 == boolop
                // arg 1 == boolop
                a = (Boolean)arguments.get(0).execute(globals, bindings);
                if (a.booleanValue())
                {
                    r = a;
                }
                else
                {
                    b = (Boolean)arguments.get(1).execute(globals, bindings);
                    if (b.booleanValue())
                    {
                        r = b;
                    }
                    else
                    {
                        r = new Boolean(false);
                    }
                }
                break;
            case S_EQ:
                r = new Boolean(equivalence(globals, bindings));
                break;
            case S_NE:
                r = new Boolean(!equivalence(globals, bindings));
                break;
            case S_GT:
            case S_LT:
            case S_GE:
            case S_LE:
                r = number_comp(globals, bindings);
                break;
            case S_ADD:
                // arg 0 == intop
                // arg 1 == intop
                arg0 = arguments.get(0).execute(globals, bindings);
                arg1 = arguments.get(1).execute(globals, bindings);

                if (arg0 instanceof String || arg1 instanceof String)
                {
                    r = arg0.toString() + arg1.toString();
                }
                else if (arg0 instanceof Point && arg1 instanceof Point)
                {
                    r = Point.add((Point)arg0, (Point)arg1);
                }
                else if (arg0 instanceof Point)
                {
                    r = Point.add((Point)arg0, (Number)arg1);
                }
                else if (arg1 instanceof Point)
                {
                    r = Point.add((Number)arg0, (Point)arg1);
                }
                else if (arg0 instanceof Double || arg1 instanceof Double)
                {
                    double t_a = ((Number)arg0).doubleValue();
                    double t_b = ((Number)arg1).doubleValue();
                    r = new Double(t_a + t_b);
                }
                else
                {
                    int t_a = ((Number)arg0).intValue();
                    int t_b = ((Number)arg1).intValue();
                    r = new Integer(t_a + t_b);
                }
                break;
            case S_SUB:
                arg0 = arguments.get(0).execute(globals, bindings);
                arg1 = arguments.get(1).execute(globals, bindings);
                
                if (arg0 instanceof Point && arg1 instanceof Point)
                {
                    r = Point.sub((Point)arg0, (Point)arg1);
                }
                else if (arg0 instanceof Point)
                {
                    r = Point.sub((Point)arg0, (Number)arg1);
                }
                else if (arg0 instanceof Double || arg1 instanceof Double)
                {
                    double t_a = ((Number)arg0).doubleValue();
                    double t_b = ((Number)arg1).doubleValue();
                    r = new Double(t_a - t_b);
                }
                else
                {
                    int t_a = ((Number)arg0).intValue();
                    int t_b = ((Number)arg1).intValue();
                    r = new Integer(t_a - t_b);
                }
                break;
            case S_MULT:
                arg0 = arguments.get(0).execute(globals, bindings);
                arg1 = arguments.get(1).execute(globals, bindings);

                if (arg0 instanceof Point && arg1 instanceof Point)
                {
                    r = Point.mult((Point)arg0, (Point)arg1);
                }
                else if (arg0 instanceof Point)
                {
                    r = Point.mult((Point)arg0, (Number)arg1);
                }
                else if (arg1 instanceof Point)
                {
                    r = Point.mult((Number)arg0, (Point)arg1);
                }
                else if (arg0 instanceof Double || arg1 instanceof Double)
                {
                    double t_a = ((Number)arg0).doubleValue();
                    double t_b = ((Number)arg1).doubleValue();
                    r = new Double(t_a * t_b);
                }
                else
                {
                    int t_a = ((Number)arg0).intValue();
                    int t_b = ((Number)arg1).intValue();
                    r = new Integer(t_a * t_b);
                }
                break;
            case S_DIV:
                arg0 = arguments.get(0).execute(globals, bindings);
                arg1 = arguments.get(1).execute(globals, bindings);

                if (arg0 instanceof Point)
                {
                    r = Point.div((Point)arg0, (Number)arg1);
                }
                else if (arg0 instanceof Double || arg1 instanceof Double)
                {
                    double t_a = ((Number)arg0).doubleValue();
                    double t_b = ((Number)arg1).doubleValue();
                    r = new Double(t_a / t_b);
                }
                else
                {
                    int t_a = ((Number)arg0).intValue();
                    int t_b = ((Number)arg1).intValue();
                    r = new Integer(t_a / t_b);
                }
                break;
            case S_POINT:
                // arg 0 == intop
                // arg 1 == intop
                arg0 = arguments.get(0).execute(globals, bindings);
                arg1 = arguments.get(1).execute(globals, bindings);
                
                Double t_a;
                Double t_b;
                
                if (arg0 instanceof Integer)
                {
                    Integer t = (Integer)arg0;
                    t_a = new Double(t.doubleValue());
                }
                else
                {
                    t_a = (Double)arg0;
                }
                if (arg1 instanceof Integer)
                {
                    Integer t = (Integer)arg1;
                    t_b = new Double(t.doubleValue());
                }
                else
                {
                    t_b = (Double)arg1;
                }
                
                r = new Point(t_a, t_b);
                break;
            case S_BANG:
                // arg 0 == boolop
                arg0 = arguments.get(0).execute(globals, bindings);
                r = new Boolean(!((Boolean)arg0).booleanValue());
                break;
            case S_NEG:
                // arg 0 == intop
                arg0 = arguments.get(0).execute(globals, bindings);
                if (arg0 instanceof Point)
                {
                    r = Point.inverse((Point)arg0);
                }
                else if (arg0 instanceof Double)
                {
                    r = new Double(-((Double)arg0).doubleValue());
                }
                else
                {
                    r = new Integer(-((Integer)arg0).intValue());
                }
                break;
            case S_VAL:
                r = value;
                break;
            case S_ID:
                if (bindings.containsKey((String)value))
                {
                    r = bindings.get((String)value);
                }
                else if (globals.containsKey((String)value))
                {
                    r = globals.get((String)value);
                }
                else
                {
                    throw new InvalidException("BlockNode: couldn't find id");
                }
                break;
            case S_PTX:
                if (bindings.containsKey((String)value))
                {
                    r = ((Point)bindings.get((String)value)).x;
                }
                else if (globals.containsKey((String)value))
                {
                    r = ((Point)globals.get((String)value)).x;
                }
                else
                {
                    throw new InvalidException("BlockNode: couldn't find id");
                }
                break;
            case S_PTY:
                if (bindings.containsKey((String)value))
                {
                    r = ((Point)bindings.get((String)value)).y;
                }
                else if (globals.containsKey((String)value))
                {
                    r = ((Point)globals.get((String)value)).y;
                }
                else
                {
                    throw new InvalidException("BlockNode: couldn't find id");
                }
                break;
            default:
                throw new InvalidException("BlockNode: case not implemented yet");
        }
        
        return r;
    }

    private boolean equivalence(HashMap<String, Object> globals, HashMap<String, Object> bindings)
        throws InvalidException
    {
        Object arg0 = arguments.get(0).execute(globals, bindings);
        Object arg1 = arguments.get(1).execute(globals, bindings);
        if (arg0 instanceof Integer)
        {
            Integer t_a = (Integer)arg0;
            return t_a.equals((Integer)arg1);
        }
        else if (arg0 instanceof Double)
        {
            Double t_a = (Double)arg0;
            return t_a.equals((Double)arg1);
        }
        else if (arg0 instanceof Boolean)
        {
            Boolean t_a = (Boolean)arg0;
            return t_a.equals((Boolean)arg1);
        }
        else if (arg0 instanceof String)
        {
            String t_a = (String)arg0;
            return t_a.equals((String)arg1);
        }
        else if (arg0 instanceof Point)
        {
            return Point.equals((Point)arg0, (Point)arg1);
        }
        else
        {
            throw new InvalidException("BlockNode: equivalence: invalid type");
        }
    }

    // S_GT, S_LT, S_GE, S_LE
    private Boolean number_comp(HashMap<String, Object> globals, HashMap<String, Object> bindings)
        throws InvalidException
    {
        Object arg0 = arguments.get(0).execute(globals, bindings);
        Object arg1 = arguments.get(1).execute(globals, bindings);
        if (arg0 instanceof Integer)
        {
            Integer t_a = (Integer)arg0;
            Integer t_b = (Integer)arg1;
            switch(op)
            {
                case S_GT:
                    return new Boolean(t_a.intValue() > t_b.intValue());
                case S_LT:
                    return new Boolean(t_a.intValue() < t_b.intValue());
                case S_GE:
                    return new Boolean(t_a.intValue() >= t_b.intValue());
                case S_LE:
                    return new Boolean(t_a.intValue() <= t_b.intValue());
                default:
                    throw new InvalidException("BlockNode: number_comp: bad op");
            }
        }
        else if (arg0 instanceof Double)
        {
            Double t_a = (Double)arg0;
            Double t_b = (Double)arg1;
            switch(op)
            {
                case S_GT:
                    return new Boolean(t_a.doubleValue() > t_b.doubleValue());
                case S_LT:
                    return new Boolean(t_a.doubleValue() < t_b.doubleValue());
                case S_GE:
                    return new Boolean(t_a.doubleValue() >= t_b.doubleValue());
                case S_LE:
                    return new Boolean(t_a.doubleValue() <= t_b.doubleValue());
                default:
                    throw new InvalidException("BlockNode: number_comp: bad op");
            }
        }
        else
        {
            throw new InvalidException("BlockNode: number_comp: invalid type");
        }
    }
}
