package cage.carl;
import cage.core.Logger;

import java.util.*;

public class Closure
{
    public enum ReturnType
    {
        R_INT,
        R_REAL,
        R_BOOL,
        R_PT,
        R_STR,
        R_NOTHING
    }

    protected BlockNode block;
    protected ReturnType returntype;
    protected ArrayList<String> param_names;
    protected ArrayList<ReturnType> param_types;
    protected HashMap<String,Object> bindings;
    
    public Closure()
    {
        block = null;
        returntype = ReturnType.R_NOTHING;
        param_names = null;
        param_types = null;
        bindings = null;
    }

    public Closure(BlockNode bl, ReturnType r, ArrayList<String> p, HashMap<String,Object> b)
    {
        block = bl;
        returntype = r;
        param_names = p;
        bindings = b;

        param_types = new ArrayList<ReturnType>();

        for (int i = 0; i < param_names.size(); i++)
        {
            param_types.add(getReturnType(bindings.get(param_names.get(i))));
        }
    }
    
    public ReturnType getReturnType()
    {
        return returntype;
    }
    
    public String getReturnTypeString()
    {
        return getReturnTypeString(returntype);
    }
    
    public String getReturnTypeClassString()
    {
        return getReturnTypeClassString(returntype);
    }
    
    public static ReturnType getReturnType(Object o)
    {
        if (o != null)
        {
            if (o instanceof Integer)
            {
                return ReturnType.R_INT;
            }
            else if (o instanceof Double)
            {
                return ReturnType.R_REAL;
            }
            else if (o instanceof Boolean)
            {
                return ReturnType.R_BOOL;
            }
            else if (o instanceof String)
            {
                return ReturnType.R_STR;
            }
            else if (o instanceof cage.carl.Point)
            {
                return ReturnType.R_PT;
            }
            else
            {
                return ReturnType.R_NOTHING;
            }
        }
        else
        {
            return ReturnType.R_NOTHING;
        }
    }

        public static ReturnType getReturnTypeFromClass(Class c)
    {
        if (c != null)
        {
            if (c.equals(Integer.class))
            {
                return ReturnType.R_INT;
            }
            else if (c.equals(Double.class))
            {
                return ReturnType.R_REAL;
            }
            else if (c.equals(Boolean.class))
            {
                return ReturnType.R_BOOL;
            }
            else if (c.equals(String.class))
            {
                return ReturnType.R_STR;
            }
            else if (c.equals(cage.carl.Point.class))
            {
                return ReturnType.R_PT;
            }
            else
            {
                return ReturnType.R_NOTHING;
            }
        }
        else
        {
            return ReturnType.R_NOTHING;
        }
    }
    
    public static String getReturnTypeString(ReturnType r)
    {
        switch (r)
        {
            case R_INT:
                return "INT";
            case R_REAL:
                return "REAL";
            case R_BOOL:
                return "BOOL";
            case R_STR:
                return "STRING";
            case R_PT:
                return "PT";
            case R_NOTHING:
                return "NOTHING";
        }
        
        return "NOTHING";
    }
    
    public static String getReturnTypeClassString(ReturnType r)
    {
        switch (r)
        {
            case R_INT:
                return "java.lang.Integer";
            case R_REAL:
                return "java.lang.Double";
            case R_BOOL:
                return "java.lang.Boolean";
            case R_STR:
                return "java.lang.String";
            case R_PT:
                return "cage.carl.Point";
            case R_NOTHING:
                return "void";
        }
        
        return "void";
    }
    
    public ArrayList<String> getParamNames()
    {
        return param_names;
    }
    
    public ArrayList<ReturnType> getParamTypes()
    {
        return param_types;
    }
    
    public Object execute(HashMap<String,Object> globals, HashMap<String,Object> params)
        throws InvalidException
    {
        HashMap<String,Object> t_bindings = new HashMap<String,Object>(bindings);
        
        if (params != null)
        {
            t_bindings.putAll(params);
        }
        
        Object r = block.execute(globals, t_bindings);
        
        if (r instanceof Returned)
        {
            return null;
        }
        else
        {
            return r;
        }
    }

    public void execute(HashMap<String,Object> globals)
    {
        try
        {
            execute(globals, null);
        }
        catch(InvalidException e)
        {
            Logger.getRootLogger().error(e.getClass().getName());
        }
    }
}