package cage.carl;

import antlr.*;
import antlr.collections.*;
import antlr.debug.misc.ASTFrame;
import java.io.*;
import java.util.*;

/**
 *  This class is only for reference and testing CARL.
 */
public class Main
{
    public static void main(String[] args)
    {
        HashMap<String,Object> globals;
        carlParser parser = new carlParser(new carlLexer(System.in));
        
        try
        {
            parser.start();
            AST t = parser.getAST();
            
            if (t != null)
            {
                //ASTFrame frame = new ASTFrame("AST", t);
                //frame.setVisible(true);
                
                carlTreeParser treeParser = new carlTreeParser();
                treeParser.start(t);
                
                globals = treeParser.getGlobals();
                
                //// test code 
                if (args[0].equals("action"))
                {
                    TestCarl.action(globals);
                }
                else if (args[0].equals("boolops"))
                {
                    TestCarl.boolops(globals);
                }
                else if (args[0].equals("call"))
                {
                    TestCarl.call(globals);
                }
                else if (args[0].equals("declares"))
                {
                    TestCarl.declares(globals);
                }
                else if (args[0].equals("function"))
                {
                    TestCarl.function(globals);
                }
                else if (args[0].equals("globals"))
                {
                    TestCarl.globals(globals);
                }
                else if (args[0].equals("ifelse"))
                {
                    TestCarl.ifelse(globals);
                }
                else if (args[0].equals("intops"))
                {
                    TestCarl.intops(globals);
                }
                else if (args[0].equals("while"))
                {
                    TestCarl.twhile(globals);
                }
                else if (args[0].equals("comment"))
                {
                    TestCarl.comment(globals);
                }
                else
                {
                    System.out.println("Unknown test type");
                }
                ////
            }
            else
            {
                System.out.println("Null tree...");
            }
        }
        catch (antlr.TokenStreamException e)
        {
            error(e.toString());
        }
        catch (antlr.RecognitionException e)
        {
            error(e.toString());
        }
        catch (InvalidException e)
        {
            error(e.toString());
        }
    }

    private static void error(String msg)
    {
        System.err.println(msg);
        System.exit(1);
    }
}
