package cage.carl;

import cage.core.Logger;
import cage.core.Cage;
import cage.core.ResourceEngine;
import cage.graphics.Sprite;
import cage.graphics.GraphicsEngine;
import cage.audio.Sound;

import java.util.Date;

//TODO: THIS IS THE ONE CLASS THAT NEEDS DOCUMENTATION FOR SURE
public class Mediator {

    private static Logger log = Logger.getRootLogger();

    //*****************************************************************
    //  Engine Support
    //*****************************************************************

    public static void printMessage(String mess) {
        System.out.println(mess);
    }

    public static void quit() {
        System.exit(0);
    }

    public static Point getScreenSize()  {
        return new Point(GraphicsEngine.getWidth(), GraphicsEngine.getHeight());
    }

    public static Integer getTime() {
        return (int)new Date().getTime();
    }

    public static void save() {
        Cage.saveState();
    }

    public static void restore() {
        Cage.restoreState();
    }


    //*****************************************************************
    //  Scenes
    //*****************************************************************

    public static void changeScene(String name) {
        Cage.changeScene(name);
    }

    public static String getCurrentScene() {
        return Cage.getCurrentScene().getName();
    }

    //*****************************************************************
    //  Animation
    //*****************************************************************
    public static void playAnimation(String animName) {
        Sprite item  = Cage.getCurrentScene().getSprite(animName);

        if (item != null && item.getAnimated()) {
            item.setPlay(true);
        }
        else {
            log.error(animName + " is not an animation and cannot be played.");
        }
    }

    public static void stopAnimation(String animName) {
        Sprite item  = Cage.getCurrentScene().getSprite(animName);

        if (item != null && item.getAnimated()) {
            item.setPlay(false);
        }
        else {
            log.error(animName + " is not an animation and cannot be played.");
        }
    }

    public static void setLoopAnimation(String animName, Boolean b) {
        Sprite item  = Cage.getCurrentScene().getSprite(animName);

        if (item != null && item.getAnimated()) {
            item.setLoop(b);
        }
        else {
            log.error(animName + " is not an animation and cannot be looped.");
        }
    }

    public static Boolean getLoopAnimation(String animName) {
        Sprite item  = Cage.getCurrentScene().getSprite(animName);

        if (item != null && item.getAnimated()) {
            return item.getLoop();
        }
        else {
            log.error(animName + " is not an animation and cannot be looped.");
        }

        return false;
    }

    //*****************************************************************
    //  Sprite
    //*****************************************************************
    public static void show(String name) {
        Sprite s = Cage.getCurrentScene().getSprite(name);

        if (s != null) {
            s.setVisible(true);
        }
    }

    public static void hide(String name) {
        Sprite s = Cage.getCurrentScene().getSprite(name);

        if (s != null) {
            s.setVisible(false);
        }
    }

    public static Boolean visible(String name) {
        Sprite s = Cage.getCurrentScene().getSprite(name);

        if (s != null) {
            return s.getVisible();
        }
        else {
            return false;
        }
    }

    public static Point getLocation(String name) {
        Sprite s = Cage.getCurrentScene().getSprite(name);

        if (s != null) {
            return new Point(s.getLocation().toString());
        }

        return new Point(0,0);
    }

    public static void setLocation(String name, Point p) {
        Sprite s = Cage.getCurrentScene().getSprite(name);

        if (s != null) {
            s.setLocation((int)p.x, (int)p.y);
        }
    }

    public static void shiftLocation(String name, Point p) {
        Sprite s = Cage.getCurrentScene().getSprite(name);

        if (s != null) {
            s.setLocation((int)(s.getX() + p.x), (int)(s.getY() + p.y));
        }
    }

    public static Point getSize(String name) {
        Sprite s = Cage.getCurrentScene().getSprite(name);

        if (s != null) {
            return new Point(s.getImage().getWidth(), s.getImage().getHeight());
        }

        return new Point(0,0);
    }

    //*****************************************************************
    //  Components
    //*****************************************************************

    public static String getText(String component) {
        try {
            return Cage.getCurrentScene().getOverlay().getComponent(component).contents.getText();
        } catch (NullPointerException ex) {
            log.error("Can't get the text on a component called " + component);
        }
        return "NULL";
    }

    public static void setText(String component, String text) {
        try {
            Cage.getCurrentScene().getOverlay().getComponent(component).contents.setText(text);
        } catch (NullPointerException ex) {
            log.error("Can't set the text on a component called " + component + " to " + text);
        }
    }


    //*****************************************************************
    //  Sound
    //*****************************************************************
    public static void playSound(String name) {
        Sound s = ResourceEngine.getSound(name);

        if (s != null) {
            s.play();
        }
        else {
            log.error("Could not find sound " + name);
        }
    }

    public static void stopSound(String name) {
        Sound s = ResourceEngine.getSound(name);

        if (s != null) {
            s.stop();
        }
        else {
            log.error("Could not find sound " + name);
        }
    }

    //*****************************************************************
    //  String
    //*****************************************************************
    public static String replace(String src, String search, String replace) {
        return src.replace(search, replace);
    }

    //*****************************************************************
    //  Math
    //*****************************************************************
    public static Double random() {
        return Math.random();
    }

    public static Double squareRoot(Double d) {
        return Math.sqrt(d);
    }

    public static Double power(Double d, Double p) {
        return Math.pow(d, p);
    }

}
