header
{
    package cage.carl;
}

class carlLexer extends Lexer;
options
{
    k=3;
    charVocabulary='\u0000'..'\u007F';
}

tokens
{
    FUNC="function";
    ACTN="action";
    TAKES="takes";
    RETURNS="returns";
    IF="if";
    ELSEIF="elseif";
    ELSE="else";
    WHILE="while";
    RETURN="return";
    INT="int";
    BOOL="bool";
    REAL="real";
    STRING="string";
    PT="point";
    FALSE="false";
    TRUE="true";
    NOTHING="nothing";
}

LPAREN   : '(';
RPAREN   : ')';
LBRACE   : '{';
RBRACE   : '}';
LBRACKET : '[';
RBRACKET : ']';
SEMI     : ';';
COLON    : ':';
COMMA    : ',';
EQUAL    : '=';
EQ       : "==";
NE       : "!=";
AND      : "&&";
OR       : "||";
LT       : '<';
LE       : "<=";
GT       : '>';
GE       : ">=";
ADD      : '+';
SUB      : '-';
MULT     : '*';
DIV      : '/';
BANG     : '!';
DOTX     : ".x";
DOTY     : ".y";

ID options {testLiterals=true;}
        : ('a'..'z'|'A'..'Z') ('a'..'z'|'A'..'Z'|'0'..'9'|'_')*;
NUMBER    : ('0'..'9')+(('.')('0'..'9')+)?;
STR     : '"' (' '|'!'|'#'..'~')* '"';
WS    : ( ' '
      | '\t'
      | '\f'
      |     ( options {generateAmbigWarnings=false;}
        : "\r\n"
        | '\r'
        | '\n'
        )
        { newline(); }
      )+
      { $setType(Token.SKIP); }
    ;
COMMENT : '#' (' '|'!'|'#'..'~'|'"')* { $setType(Token.SKIP); };

class carlParser extends Parser;
options
{
    k=3;
    buildAST=true;
    defaultErrorHandler=false;
}

tokens
{
    SCRIPT;
    DECLS;
    FUNCS;
    BLOCK;
    CALL;
    ASSIGN;
    ASSIGN_DOTX;
    ASSIGN_DOTY;
    NEG;
    POINT;
    ELIFS;
}

start
    : script
    ;

script
    : declarations functions
        { #script = #([SCRIPT,"SCRIPT"], #script); }
    ;

declarations
    : (decl)*
        { #declarations = #([DECLS,"DECLS"], #declarations); }
    ;
    
decl
    : ID^ COLON! type
    ;
    
type
    : INT
    | REAL
    | BOOL
    | PT
    | STRING
    | NOTHING
    ;

functions
    : (action | function)*
        { #functions = #([FUNCS,"FUNCS"], #functions); }
    ;
    
action
    : ACTN^ ID block
    ;
    
function
    : FUNC^ ID TAKES! (NOTHING! | declarations) RETURNS! type block
    ;
    
block
    : LBRACE! block_innards RBRACE!
    ;
    
block_innards
    : (decl | statement)*
        { #block_innards = #([BLOCK,"BLOCK"], #block_innards); }
    ;
    
statement
    : if_statement
    | while_statement
    | command
    ;
    
if_statement
    : IF^ expression block (elseif_statements)? (ELSE! block)?
    ;
    
elseif_statements
    : (elseif_statement)+
        { #elseif_statements = #([ELIFS,"ELIFS"], #elseif_statements); }
    ;

elseif_statement
    : ELSEIF^ expression block
    ;
    
while_statement
    : WHILE^ expression block
    ;

command
    : call SEMI!
    | assign SEMI!
    | return_statement SEMI!
    ;

call
    : ID (expression)*
        { #call = #([CALL,"CALL"], #call); }
    ;
    
assign
    : ID EQUAL! expression
        { #assign = #([ASSIGN,"ASSIGN"], #assign); }
    | ID DOTX! EQUAL! expression
        { #assign = #([ASSIGN_DOTX,"ASSIGN_DOTX"], #assign); }
    | ID DOTY! EQUAL! expression
        { #assign = #([ASSIGN_DOTY,"ASSIGN_DOTY"], #assign); }
    ;
    
return_statement
    : RETURN^ (expression)?
    ;
    
expression
    : BANG^ stuff
    | SUB! stuff
        { #expression = #([NEG,"NEG"], #expression); }
    | stuff
    ;

stuff
    : item
    | LPAREN! boolop RPAREN!
    | LBRACKET! call RBRACKET!
    ;
    
boolop
    : comp ((AND^ | OR^) comp)*
    ;
    
comp
    : addop ((EQ^ | NE^ | LT^ | GT^ | LE^ | GE^) addop)?
    ;
    
addop
    : multop ((ADD^ | SUB^) multop)*
    ;
    
multop
    : expression ((MULT^ | DIV^) expression)*
    ;
    
item
    : ID
    | NUMBER
    | TRUE
    | FALSE
    | point
    | pointdot
    | STR
    ;
    
point
    : LT! expression COMMA! expression GT!
        { #point = #([POINT,"POINT"], #point); }
    ;
    
pointdot
    : ID DOTX^
    | ID DOTY^
    ;

{
    import java.util.*;
}
class carlTreeParser extends TreeParser;
options
{
    importVocab=carlLexer;
}

{
    HashMap<String,Object> globals = new HashMap<String,Object>();
    
    int statementnumber = 0;
    
    public HashMap<String,Object> getGlobals()
    {
        return globals;
    }

    public void populateGlobals(HashMap<String,Object> g)
    {
        globals.putAll(g);
    }

    private Closure.ReturnType typeCheck(String a, Closure.ReturnType r, HashMap<String,Object> bindings)
        throws InvalidException
    {
        Closure.ReturnType atype;
    
        if (!bindings.containsKey(a))
        {
            if (!globals.containsKey(a))
            {
                throw new InvalidException("id not found: " + a);
            }
            
            atype = Closure.getReturnType(globals.get(a));
    
            if (r != atype)
            {
                throw new InvalidException("Type mismatch, expected: " + Closure.getReturnTypeString(r) + " found: " + Closure.getReturnTypeString(atype));
            }
        }
        else
        {
            atype = Closure.getReturnType(bindings.get(a));
            
            if (r != atype)
            {
                throw new InvalidException("Type mismatch, expected: " + Closure.getReturnTypeString(r) + " found: " + Closure.getReturnTypeString(atype));
            }
        }
        
        return r;
    }

    private BlockNode doBoolop(BlockNode a, BlockNode b, BlockNode.Operation op)
        throws InvalidException
    {
        if (a.getReturnType() != Closure.ReturnType.R_BOOL ||
            b.getReturnType() != Closure.ReturnType.R_BOOL)
        {
            switch(op)
            {
                case S_AND:
                    throw new InvalidException("Boolean operator '&&' requires two boolean operands");
                case S_OR:
                    throw new InvalidException("Boolean operator '||' requires two boolean operands");
                default:
                    throw new InvalidException("Unexpected boolean operator");
            }
        }
        ArrayList<BlockNode> args = new ArrayList<BlockNode>();
        args.add(a);
        args.add(b);
        return new BlockNode(op,null,args,Closure.ReturnType.R_BOOL);
    }

    private BlockNode doCompare(BlockNode a, BlockNode b, BlockNode.Operation op)
        throws InvalidException
    {
        if (a.getReturnType() != b.getReturnType())
        {
            switch(op)
            {
                case S_GT:
                    throw new InvalidException("Comparison operator '>' requires two operands of the same type");
                case S_LT:
                    throw new InvalidException("Comparison operator '<' requires two operands of the same type");
                case S_GE:
                    throw new InvalidException("Comparison operator '>=' requires two operands of the same type");
                case S_LE:
                    throw new InvalidException("Comparison operator '<=' requires two operands of the same type");
                default:
                    throw new InvalidException("Unexpected comparison operator");
            }
        }
        
        if (op != BlockNode.Operation.S_EQ &&
            op != BlockNode.Operation.S_NE)
        {
            if (a.getReturnType() == Closure.ReturnType.R_PT ||
                a.getReturnType() == Closure.ReturnType.R_STR ||
                a.getReturnType() == Closure.ReturnType.R_BOOL)
            {
                throw new InvalidException("Points, Strings, and Booleans are only comparable with == and !=");
            }
        }
        
        ArrayList<BlockNode> args = new ArrayList<BlockNode>();
        args.add(a);
        args.add(b);
        return new BlockNode(op,null,args,Closure.ReturnType.R_BOOL);
    }

    private BlockNode doIntop(BlockNode a, BlockNode b, BlockNode.Operation op)
        throws InvalidException
    {
        Closure.ReturnType aret = a.getReturnType();
        Closure.ReturnType bret = b.getReturnType();
        if (aret == Closure.ReturnType.R_BOOL ||
            bret == Closure.ReturnType.R_BOOL)
        {
            throw new InvalidException("expected type int, real, point; found bool");
        }

        if ((aret == Closure.ReturnType.R_STR ||
            bret == Closure.ReturnType.R_STR) &&
            op != BlockNode.Operation.S_ADD)
        {
            throw new InvalidException("Strings may only be concatenated (+)");
        }
        
        if (aret == Closure.ReturnType.R_PT &&
            bret == Closure.ReturnType.R_PT &&
            op == BlockNode.Operation.S_DIV)
        {
            throw new InvalidException("Points cannot be divided");
        }
    
        ArrayList<BlockNode> args = new ArrayList<BlockNode>();
        args.add(a);
        args.add(b);

        if (aret != bret)
        {
            if (aret == Closure.ReturnType.R_STR ||
                bret == Closure.ReturnType.R_STR)
            {
                return new BlockNode(op,null,args,Closure.ReturnType.R_STR);
            }

            if (aret == Closure.ReturnType.R_PT)
            {
                return new BlockNode(op,null,args,Closure.ReturnType.R_PT);
            }
            else if (bret == Closure.ReturnType.R_PT)
            {
                if (op == BlockNode.Operation.S_ADD ||
                    op == BlockNode.Operation.S_MULT)
                {
                    return new BlockNode(op,null,args,Closure.ReturnType.R_PT);
                }
                else
                {
                    throw new InvalidException("Points cannot divide or be subtracted from other values");
                }
            }

            if (aret == Closure.ReturnType.R_REAL ||
                bret == Closure.ReturnType.R_REAL)
            {
                return new BlockNode(op,null,args,Closure.ReturnType.R_REAL);
            }
            
            String error1 = "Incompatible types (" + Closure.getReturnTypeString(aret);
            String error2 = Closure.getReturnTypeString(bret) + ")";
            
            switch(op)
            {
                case S_ADD:
                    throw new InvalidException(error1 + " + " + error2);
                case S_SUB:
                    throw new InvalidException(error1 + " - " + error2);
                case S_MULT:
                    throw new InvalidException(error1 + " * " + error2);
                case S_DIV:
                    throw new InvalidException(error1 + " / " + error2);
                default:
                    throw new InvalidException("Unexpected math operator");
            }
        }

        return new BlockNode(op,null,args,aret);
    }
    
    private Closure.ReturnType scanReturnType(ArrayList<BlockNode> b, Closure.ReturnType returntype)
        throws InvalidException
    {
        ListIterator<BlockNode> iter = b.listIterator(0);
        
        while(iter.hasNext())
        {
            BlockNode temp = iter.next();
            Closure.ReturnType r;
            
            if (temp == null)
            {
                continue;
            }
            else
            {
                r = temp.getReturnType();
            }
            
            if (r != Closure.ReturnType.R_NOTHING)
            {
                if (r == returntype)
                {
                    return returntype;
                }
                else
                {
                    throw new InvalidException("function returns " + Closure.getReturnTypeString(r) + ", declared to return: " + Closure.getReturnTypeString(returntype));
                }
            }
        }
        
        return Closure.ReturnType.R_NOTHING;
    }
}

start
    throws InvalidException
{ ArrayList<String> ignore; }
    : #(SCRIPT (ignore=decls[globals])? funcs)
    ;

decls [HashMap<String,Object> bindings] returns [ ArrayList<String> r=new ArrayList<String>(); ]
{ String t; }
    : #(DECLS (t=decl[bindings] { r.add(t); })*)
    ;

decl [HashMap<String,Object> bindings] returns [ String r=new String(); ]
{ Object t; }
    : #(id:ID t=type_obj)
        {
            bindings.put(id.getText(), t);
            r = id.getText();
        }
    ;

type_obj returns [Object r=new Object();]
    : INT    { r = new Integer(0); }
    | REAL   { r = new Double(0); }
    | BOOL   { r = new Boolean(false); }
    | PT     { r = new Point(); }
    | STRING { r = new String(); }
    | NOTHING { r = null; }
    ;
    
type returns [Closure.ReturnType r = Closure.ReturnType.R_NOTHING;]
    : INT    { r = Closure.ReturnType.R_INT; }
    | REAL   { r = Closure.ReturnType.R_REAL; }
    | BOOL   { r = Closure.ReturnType.R_BOOL; }
    | PT     { r = Closure.ReturnType.R_PT; }
    | STRING { r = Closure.ReturnType.R_STR; }
    | NOTHING { r = Closure.ReturnType.R_NOTHING; }
    ;

funcs
    throws InvalidException
    : #(FUNCS (func)*)
    ;

func
    throws InvalidException
{ BlockNode b; Closure.ReturnType returntype; HashMap<String,Object> bindings = new HashMap<String,Object>(); ArrayList<String> pnames = null; }
    : #(ACTN id:ID bl2:.)
        {
            statementnumber = 0;
            try
            {
                b=block(bl2,bindings,Closure.ReturnType.R_NOTHING);
            }
            catch (InvalidException e)
            {
                throw new InvalidException("Error in action: " + id.getText() + "\n\tAt statement: " + statementnumber + "\n\t" + e);
            }
            globals.put(id.getText(),new Closure(b,Closure.ReturnType.R_NOTHING,new ArrayList<String>(),new HashMap<String,Object>())); 
        }
    | #(FUNC ie:ID (pnames=decls[bindings])? returntype=type bl:.)
        {
            if (pnames == null)
            {
                pnames = new ArrayList<String>();
            }
                        
            statementnumber = 0;
            try
            {
                b=block(bl,bindings,returntype);
                
                if (b.getReturnType() != returntype)
                {
                    throw new InvalidException("function returns " + Closure.getReturnTypeString(b.getReturnType()) + ", declared to return: " + Closure.getReturnTypeString(returntype));
                }
            }
            catch (InvalidException e)
            {
                throw new InvalidException("Error in function: " + ie.getText() + "\n\tAt statement: " + statementnumber + "\n\t" + e);
            }
            
            globals.put(ie.getText(),new Closure(b,returntype,pnames,bindings));
        }
    ;

block [HashMap<String,Object> bindings, Closure.ReturnType returntype] returns [BlockNode r=null;]
    throws InvalidException
{ ArrayList<BlockNode> b; }
    : #(BLOCK b=block_innards[bindings,returntype])
        {
            r = new BlockNode(BlockNode.Operation.S_BLOCK,null,b,scanReturnType(b,returntype));
        }
    ;

block_innards [HashMap<String,Object> bindings, Closure.ReturnType returntype] returns [ArrayList<BlockNode> r=new ArrayList<BlockNode>();]
    throws InvalidException
{ BlockNode s; }
    : (
        (s=statement[bindings,returntype])
        {
            if (s != null)
            {
                r.add(s);
            }
        }
      )*
    ;

statement [HashMap<String,Object> bindings, Closure.ReturnType returntype] returns [BlockNode r=null;]
    throws InvalidException
{ Object t; BlockNode e,b,b2 = null; ArrayList<BlockNode> el = null; }
    : #(id:ID t=type_obj)
        {
            statementnumber += 1;
            if (bindings.containsKey(id.getText()))
            {
                throw new InvalidException("variable '" + id.getText() + "' redeclared");
            }
            bindings.put(id.getText(),t);
        }
    | #(IF e=boolop[bindings] b=block[bindings,returntype] (el=elseifs[bindings,returntype])? (b2=block[bindings,returntype])?)
        {
            ArrayList<BlockNode> args = new ArrayList<BlockNode>();
            
            if (e.getReturnType() != Closure.ReturnType.R_BOOL)
            {
                throw new InvalidException("if statement requires boolean check");
            }
            
            args.add(e);
            args.add(b);
            
            boolean returns = (b.getReturnType() == returntype);
            
            if (el != null)
            {
                for (int i = 0; i < el.size(); i += 2)
                {
                    // These come in pairs - expression, then block
                    BlockNode be = el.get(i);
                    args.add(be);
                    
                    BlockNode bb = el.get(i+1);
                    args.add(bb);
                    
                    returns = returns && (bb.getReturnType() == returntype);
                }
            }
            
            if (b2 != null)
            {
                args.add(b2);
                
                returns = returns && (b2.getReturnType() == returntype);
            }
            
            if (returns)
            {
                r = new BlockNode(BlockNode.Operation.S_IF,null,args,returntype);
            }
            else
            {
                r = new BlockNode(BlockNode.Operation.S_IF,null,args,Closure.ReturnType.R_NOTHING);
            }
        }
    | #(WHILE e=boolop[bindings] b=block[bindings,returntype])
        {
            ArrayList<BlockNode> args = new ArrayList<BlockNode>();
            
            if (e.getReturnType() != Closure.ReturnType.R_BOOL)
            {
                throw new InvalidException("while statement requires boolean check");
            }
            
            args.add(e);
            args.add(b);
            
            if (b.getReturnType() == returntype)
            {
                r = new BlockNode(BlockNode.Operation.S_WHILE,null,args,returntype);
            }
            else if (b.getReturnType() == Closure.ReturnType.R_NOTHING)
            {
                r = new BlockNode(BlockNode.Operation.S_WHILE,null,args,Closure.ReturnType.R_NOTHING);
            }
            else
            {
                throw new InvalidException("function returns " + Closure.getReturnTypeString(b.getReturnType()) + " , declared to return: " + Closure.getReturnTypeString(returntype));
            }
        }
    | r=call[bindings,true]
    | r=assign[bindings]
    | #(RETURN (rstatement:.)?)
        {
            statementnumber += 1;
            ArrayList<BlockNode> args = new ArrayList<BlockNode>();
            
            if (rstatement != null)
            {
                e = boolop(rstatement, bindings);
                if (e.getReturnType() != returntype)
                {
                    throw new InvalidException("function returns " + Closure.getReturnTypeString(e.getReturnType()) + ", declared to return: " + Closure.getReturnTypeString(returntype));
                }
                args.add(e);
            }
            else if (returntype != Closure.ReturnType.R_NOTHING)
            {
                throw new InvalidException("function returns nothing, declared to return: " + Closure.getReturnTypeString(returntype));
            }
            
            r = new BlockNode(BlockNode.Operation.S_RETURN,null,args,returntype);
        }
    ;

assign [HashMap<String,Object> bindings] returns [BlockNode r=null;]
    throws InvalidException
{ BlockNode b; }
    : #(ASSIGN id:ID b=boolop[bindings])
        {
            statementnumber += 1;
            Closure.ReturnType type = b.getReturnType();
            String i = id.getText();
            
            ArrayList<BlockNode> args = new ArrayList<BlockNode>();
            args.add(new BlockNode(BlockNode.Operation.S_ID,i,null,typeCheck(i,type,bindings)));
            args.add(b);
            r = new BlockNode(BlockNode.Operation.S_ASSIGN,null,args,Closure.ReturnType.R_NOTHING);
        }
    | #(ASSIGN_DOTX ia:ID b=boolop[bindings])
        {
            statementnumber += 1;
            Closure.ReturnType type = b.getReturnType();

            if (type != Closure.ReturnType.R_REAL &&
                type != Closure.ReturnType.R_INT)
            {
                throw new InvalidException("Type mismatch, expected: real found: " + Closure.getReturnTypeString(type));
            }

            String i = ia.getText();

            ArrayList<BlockNode> args = new ArrayList<BlockNode>();
            args.add(new BlockNode(BlockNode.Operation.S_ID,i,null,Closure.ReturnType.R_PT));
            args.add(b);
            r = new BlockNode(BlockNode.Operation.S_ASSIGN_DOTX,null,args,Closure.ReturnType.R_NOTHING);
        }
    | #(ASSIGN_DOTY ib:ID b=boolop[bindings])
        {
            statementnumber += 1;
            Closure.ReturnType type = b.getReturnType();

            if (type != Closure.ReturnType.R_REAL &&
                type != Closure.ReturnType.R_INT)
            {
                throw new InvalidException("Type mismatch, expected: real found: " + Closure.getReturnTypeString(type));
            }

            String i = ib.getText();

            ArrayList<BlockNode> args = new ArrayList<BlockNode>();
            args.add(new BlockNode(BlockNode.Operation.S_ID,i,null,Closure.ReturnType.R_PT));
            args.add(b);
            r = new BlockNode(BlockNode.Operation.S_ASSIGN_DOTY,null,args,Closure.ReturnType.R_NOTHING);
        }
    ;

call [HashMap<String,Object> bindings,boolean individual] returns [BlockNode r=null;]
    throws InvalidException
{ ArrayList<BlockNode> e; }
    : #(CALL id:ID e=exprs[bindings])
        {
            if (individual)
            {
                statementnumber += 1;
            }
            if (!globals.containsKey(id.getText()))
            {
                throw new InvalidException("function called without being defined: " + id.getText());
            }
            
            Closure func = (Closure)globals.get(id.getText());
            ArrayList<Closure.ReturnType> types = func.getParamTypes();
            Closure.ReturnType returntype = func.getReturnType();
            
            if (e.size() != types.size())
            {
                throw new InvalidException("function " + id.getText() + " called with " + e.size() + " parameters; expects " + types.size());
            }
        
            for (int i = 0; i < e.size(); i++)
            {
                BlockNode b = e.get(i);
                if (b.getReturnType() != types.get(i))
                {
                    throw new InvalidException("function " + id.getText() + " called with mismatched parameter type(s)");
                }
            }
            
            e.add(0, new BlockNode(func));

            r = new BlockNode(BlockNode.Operation.S_CALL,null,e,returntype);
        }
    ;
    
elseifs [HashMap<String,Object> bindings, Closure.ReturnType returntype] returns [ArrayList<BlockNode> r = new ArrayList<BlockNode>();]
    throws InvalidException
    : #(ELIFS (elseif[bindings,returntype,r])+)
    ;
    
elseif [HashMap<String,Object> bindings, Closure.ReturnType returntype, ArrayList<BlockNode> nodes]
    throws InvalidException
{ BlockNode c,b; }
    : #(ELSEIF c=boolop[bindings] b=block[bindings,returntype])
        {
            if (c.getReturnType() != Closure.ReturnType.R_BOOL)
            {
                throw new InvalidException("elseif statement requires boolean check");
            }
            nodes.add(c);
            nodes.add(b);
        }
    ;

exprs [HashMap<String,Object> bindings] returns [ArrayList<BlockNode> r=new ArrayList<BlockNode>();]
    throws InvalidException
{ BlockNode e; }
    : ((e=boolop[bindings]) { r.add(e); })*
    ;

boolop [HashMap<String,Object> bindings] returns [BlockNode r=null;]
    throws InvalidException
{ BlockNode a,b; }
    : #(AND a=boolop[bindings] b=boolop[bindings]) { r = doBoolop(a,b,BlockNode.Operation.S_AND); }
    | #(OR a=boolop[bindings] b=boolop[bindings])  { r = doBoolop(a,b,BlockNode.Operation.S_OR);  }
    | #(EQ a=intop[bindings] b=intop[bindings])    { r = doCompare(a,b,BlockNode.Operation.S_EQ); }
    | #(NE a=intop[bindings] b=intop[bindings])    { r = doCompare(a,b,BlockNode.Operation.S_NE); }
    | #(LT a=intop[bindings] b=intop[bindings])    { r = doCompare(a,b,BlockNode.Operation.S_LT); }
    | #(GT a=intop[bindings] b=intop[bindings])    { r = doCompare(a,b,BlockNode.Operation.S_GT); }
    | #(LE a=intop[bindings] b=intop[bindings])    { r = doCompare(a,b,BlockNode.Operation.S_LE); }
    | #(GE a=intop[bindings] b=intop[bindings])    { r = doCompare(a,b,BlockNode.Operation.S_GE); }
    | #(BANG a=boolop[bindings])
        {
            if (a.getReturnType() != Closure.ReturnType.R_BOOL)
            {
                throw new InvalidException("! operator only negates Boolean");
            }
            ArrayList<BlockNode> args = new ArrayList<BlockNode>();
            args.add(a);
            r = new BlockNode(BlockNode.Operation.S_BANG,null,args,Closure.ReturnType.R_BOOL);
        }
    | TRUE  { r = new BlockNode(new Boolean(true)); }
    | FALSE { r = new BlockNode(new Boolean(false)); }
    | r=intop[bindings]
    ;

intop [HashMap<String,Object> bindings] returns [BlockNode r=null;]
    throws InvalidException
{ BlockNode a,b; }
    : #(ADD a=intop[bindings] b=intop[bindings])  { r = doIntop(a,b,BlockNode.Operation.S_ADD);  }
    | #(SUB a=intop[bindings] b=intop[bindings])  { r = doIntop(a,b,BlockNode.Operation.S_SUB);   }
    | #(MULT a=intop[bindings] b=intop[bindings]) { r = doIntop(a,b,BlockNode.Operation.S_MULT); }
    | #(DIV a=intop[bindings] b=intop[bindings])  { r = doIntop(a,b,BlockNode.Operation.S_DIV);  }
    | #(NEG a=intop[bindings])
        {
            Closure.ReturnType aret = a.getReturnType();
            if (aret != Closure.ReturnType.R_INT &&
                aret != Closure.ReturnType.R_REAL  &&
                aret != Closure.ReturnType.R_PT)
            {
                throw new InvalidException("- operator only negates Int, Real, and Point");
            }
            ArrayList<BlockNode> args = new ArrayList<BlockNode>();
            args.add(a);
            r = new BlockNode(BlockNode.Operation.S_NEG,null,args,aret);
        }
    | n:NUMBER
        {
            String num = n.getText();
            if (num.contains("."))
            {
                r = new BlockNode(new Double(num));
            }
            else
            {
                r = new BlockNode(new Integer(num));
            }
        }
    | r=call[bindings,false]
    | r=item[bindings]
    ;

item [HashMap<String,Object> bindings] returns [BlockNode r=null;]
    throws InvalidException
{ BlockNode a,b; }
    : i:ID
        {
            String id = i.getText();
            if (bindings.containsKey(id))
            {
                r = new BlockNode(BlockNode.Operation.S_ID,id,null,Closure.getReturnType(bindings.get(id)));
            }
            else if (globals.containsKey(id))
            {
                r = new BlockNode(BlockNode.Operation.S_ID,id,null,Closure.getReturnType(globals.get(id)));
            }
            else
            {
                throw new InvalidException("variable used without being declared: " + id);
            }
        }
    | #(POINT a=intop[bindings] b=intop[bindings])
        {
            Closure.ReturnType aret = a.getReturnType();
            Closure.ReturnType bret = b.getReturnType();
            if ((aret != Closure.ReturnType.R_INT &&
                    aret != Closure.ReturnType.R_REAL) ||
                (bret != Closure.ReturnType.R_INT &&
                    bret != Closure.ReturnType.R_REAL))
            {
                throw new InvalidException("Point may only contain Int or Real");
            }

            ArrayList<BlockNode> args = new ArrayList<BlockNode>();
            args.add(a);
            args.add(b);

            r = new BlockNode(BlockNode.Operation.S_POINT,null,args,Closure.ReturnType.R_PT);
        }
    | #(DOTX ix:ID)
        {
            String id = ix.getText();
            if (bindings.containsKey(id))
            {
                if (Closure.getReturnType(bindings.get(id)) == Closure.ReturnType.R_PT)
                {
                    r = new BlockNode(BlockNode.Operation.S_PTX,id,null,Closure.ReturnType.R_REAL);
                }
                else
                {
                    throw new InvalidException("dot operator expects type Point");
                }
            }
            else if (globals.containsKey(id))
            {
                if (Closure.getReturnType(globals.get(id)) == Closure.ReturnType.R_PT)
                {
                    r = new BlockNode(BlockNode.Operation.S_PTX,id,null,Closure.ReturnType.R_REAL);
                }
                else
                {
                    throw new InvalidException("dot operator expects type Point");
                }
            }
            else
            {
                throw new InvalidException("variable used without being declared: " + id);
            }
        }
    | #(DOTY iy:ID)
        {
            String id = iy.getText();
            if (bindings.containsKey(id))
            {
                if (Closure.getReturnType(bindings.get(id)) == Closure.ReturnType.R_PT)
                {
                    r = new BlockNode(BlockNode.Operation.S_PTY,id,null,Closure.ReturnType.R_REAL);
                }
                else
                {
                    throw new InvalidException("dot operator expects type Point");
                }
            }
            else if (globals.containsKey(id))
            {
                if (Closure.getReturnType(globals.get(id)) == Closure.ReturnType.R_PT)
                {
                    r = new BlockNode(BlockNode.Operation.S_PTY,id,null,Closure.ReturnType.R_REAL);
                }
                else
                {
                    throw new InvalidException("dot operator expects type Point");
                }
            }
            else
            {
                throw new InvalidException("variable used without being declared: " + id);
            }
        }
    | s:STR
        {
            String t_s = s.getText();
            r = new BlockNode(t_s.substring(1,t_s.length()-1).replace("\\n","\n"));
        }
    ;

