// $ANTLR 2.7.7 (20070330): "carl.g" -> "carlTreeParser.java"$

    package cage.carl;

import antlr.TreeParser;
import antlr.Token;
import antlr.collections.AST;
import antlr.RecognitionException;
import antlr.ANTLRException;
import antlr.NoViableAltException;
import antlr.MismatchedTokenException;
import antlr.SemanticException;
import antlr.collections.impl.BitSet;
import antlr.ASTPair;
import antlr.collections.impl.ASTArray;

    import java.util.*;


public class carlTreeParser extends antlr.TreeParser       implements carlTreeParserTokenTypes
 {

    HashMap<String,Object> globals = new HashMap<String,Object>();
    
    int statementnumber = 0;
    
    public HashMap<String,Object> getGlobals()
    {
        return globals;
    }

    public void populateGlobals(HashMap<String,Object> g)
    {
        globals.putAll(g);
    }

    private Closure.ReturnType typeCheck(String a, Closure.ReturnType r, HashMap<String,Object> bindings)
        throws InvalidException
    {
        Closure.ReturnType atype;
    
        if (!bindings.containsKey(a))
        {
            if (!globals.containsKey(a))
            {
                throw new InvalidException("id not found: " + a);
            }
            
            atype = Closure.getReturnType(globals.get(a));
    
            if (r != atype)
            {
                throw new InvalidException("Type mismatch, expected: " + Closure.getReturnTypeString(r) + " found: " + Closure.getReturnTypeString(atype));
            }
        }
        else
        {
            atype = Closure.getReturnType(bindings.get(a));
            
            if (r != atype)
            {
                throw new InvalidException("Type mismatch, expected: " + Closure.getReturnTypeString(r) + " found: " + Closure.getReturnTypeString(atype));
            }
        }
        
        return r;
    }

    private BlockNode doBoolop(BlockNode a, BlockNode b, BlockNode.Operation op)
        throws InvalidException
    {
        if (a.getReturnType() != Closure.ReturnType.R_BOOL ||
            b.getReturnType() != Closure.ReturnType.R_BOOL)
        {
            switch(op)
            {
                case S_AND:
                    throw new InvalidException("Boolean operator '&&' requires two boolean operands");
                case S_OR:
                    throw new InvalidException("Boolean operator '||' requires two boolean operands");
                default:
                    throw new InvalidException("Unexpected boolean operator");
            }
        }
        ArrayList<BlockNode> args = new ArrayList<BlockNode>();
        args.add(a);
        args.add(b);
        return new BlockNode(op,null,args,Closure.ReturnType.R_BOOL);
    }

    private BlockNode doCompare(BlockNode a, BlockNode b, BlockNode.Operation op)
        throws InvalidException
    {
        if (a.getReturnType() != b.getReturnType())
        {
            switch(op)
            {
                case S_GT:
                    throw new InvalidException("Comparison operator '>' requires two operands of the same type");
                case S_LT:
                    throw new InvalidException("Comparison operator '<' requires two operands of the same type");
                case S_GE:
                    throw new InvalidException("Comparison operator '>=' requires two operands of the same type");
                case S_LE:
                    throw new InvalidException("Comparison operator '<=' requires two operands of the same type");
                default:
                    throw new InvalidException("Unexpected comparison operator");
            }
        }
        
        if (op != BlockNode.Operation.S_EQ &&
            op != BlockNode.Operation.S_NE)
        {
            if (a.getReturnType() == Closure.ReturnType.R_PT ||
                a.getReturnType() == Closure.ReturnType.R_STR ||
                a.getReturnType() == Closure.ReturnType.R_BOOL)
            {
                throw new InvalidException("Points, Strings, and Booleans are only comparable with == and !=");
            }
        }
        
        ArrayList<BlockNode> args = new ArrayList<BlockNode>();
        args.add(a);
        args.add(b);
        return new BlockNode(op,null,args,Closure.ReturnType.R_BOOL);
    }

    private BlockNode doIntop(BlockNode a, BlockNode b, BlockNode.Operation op)
        throws InvalidException
    {
        Closure.ReturnType aret = a.getReturnType();
        Closure.ReturnType bret = b.getReturnType();
        if (aret == Closure.ReturnType.R_BOOL ||
            bret == Closure.ReturnType.R_BOOL)
        {
            throw new InvalidException("expected type int, real, point; found bool");
        }

        if ((aret == Closure.ReturnType.R_STR ||
            bret == Closure.ReturnType.R_STR) &&
            op != BlockNode.Operation.S_ADD)
        {
            throw new InvalidException("Strings may only be concatenated (+)");
        }
        
        if (aret == Closure.ReturnType.R_PT &&
            bret == Closure.ReturnType.R_PT &&
            op == BlockNode.Operation.S_DIV)
        {
            throw new InvalidException("Points cannot be divided");
        }
    
        ArrayList<BlockNode> args = new ArrayList<BlockNode>();
        args.add(a);
        args.add(b);

        if (aret != bret)
        {
            if (aret == Closure.ReturnType.R_STR ||
                bret == Closure.ReturnType.R_STR)
            {
                return new BlockNode(op,null,args,Closure.ReturnType.R_STR);
            }

            if (aret == Closure.ReturnType.R_PT)
            {
                return new BlockNode(op,null,args,Closure.ReturnType.R_PT);
            }
            else if (bret == Closure.ReturnType.R_PT)
            {
                if (op == BlockNode.Operation.S_ADD ||
                    op == BlockNode.Operation.S_MULT)
                {
                    return new BlockNode(op,null,args,Closure.ReturnType.R_PT);
                }
                else
                {
                    throw new InvalidException("Points cannot divide or be subtracted from other values");
                }
            }

            if (aret == Closure.ReturnType.R_REAL ||
                bret == Closure.ReturnType.R_REAL)
            {
                return new BlockNode(op,null,args,Closure.ReturnType.R_REAL);
            }
            
            String error1 = "Incompatible types (" + Closure.getReturnTypeString(aret);
            String error2 = Closure.getReturnTypeString(bret) + ")";
            
            switch(op)
            {
                case S_ADD:
                    throw new InvalidException(error1 + " + " + error2);
                case S_SUB:
                    throw new InvalidException(error1 + " - " + error2);
                case S_MULT:
                    throw new InvalidException(error1 + " * " + error2);
                case S_DIV:
                    throw new InvalidException(error1 + " / " + error2);
                default:
                    throw new InvalidException("Unexpected math operator");
            }
        }

        return new BlockNode(op,null,args,aret);
    }
    
    private Closure.ReturnType scanReturnType(ArrayList<BlockNode> b, Closure.ReturnType returntype)
        throws InvalidException
    {
        ListIterator<BlockNode> iter = b.listIterator(0);
        
        while(iter.hasNext())
        {
            BlockNode temp = iter.next();
            Closure.ReturnType r;
            
            if (temp == null)
            {
                continue;
            }
            else
            {
                r = temp.getReturnType();
            }
            
            if (r != Closure.ReturnType.R_NOTHING)
            {
                if (r == returntype)
                {
                    return returntype;
                }
                else
                {
                    throw new InvalidException("function returns " + Closure.getReturnTypeString(r) + ", declared to return: " + Closure.getReturnTypeString(returntype));
                }
            }
        }
        
        return Closure.ReturnType.R_NOTHING;
    }
public carlTreeParser() {
	tokenNames = _tokenNames;
}

	public final void start(AST _t) throws RecognitionException, InvalidException {
		
		AST start_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		ArrayList<String> ignore;
		
		try {      // for error handling
			AST __t101 = _t;
			AST tmp1_AST_in = (AST)_t;
			match(_t,SCRIPT);
			_t = _t.getFirstChild();
			{
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case DECLS:
			{
				ignore=decls(_t,globals);
				_t = _retTree;
				break;
			}
			case FUNCS:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
			}
			funcs(_t);
			_t = _retTree;
			_t = __t101;
			_t = _t.getNextSibling();
		}
		catch (RecognitionException ex) {
			reportError(ex);
			if (_t!=null) {_t = _t.getNextSibling();}
		}
		_retTree = _t;
	}
	
	public final ArrayList<String>  decls(AST _t,
		HashMap<String,Object> bindings
	) throws RecognitionException {
		 ArrayList<String> r=new ArrayList<String>(); ;
		
		AST decls_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		String t;
		
		try {      // for error handling
			AST __t104 = _t;
			AST tmp2_AST_in = (AST)_t;
			match(_t,DECLS);
			_t = _t.getFirstChild();
			{
			_loop106:
			do {
				if (_t==null) _t=ASTNULL;
				if ((_t.getType()==ID)) {
					t=decl(_t,bindings);
					_t = _retTree;
					r.add(t);
				}
				else {
					break _loop106;
				}
				
			} while (true);
			}
			_t = __t104;
			_t = _t.getNextSibling();
		}
		catch (RecognitionException ex) {
			reportError(ex);
			if (_t!=null) {_t = _t.getNextSibling();}
		}
		_retTree = _t;
		return r;
	}
	
	public final void funcs(AST _t) throws RecognitionException, InvalidException {
		
		AST funcs_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		try {      // for error handling
			AST __t112 = _t;
			AST tmp3_AST_in = (AST)_t;
			match(_t,FUNCS);
			_t = _t.getFirstChild();
			{
			_loop114:
			do {
				if (_t==null) _t=ASTNULL;
				if ((_t.getType()==FUNC||_t.getType()==ACTN)) {
					func(_t);
					_t = _retTree;
				}
				else {
					break _loop114;
				}
				
			} while (true);
			}
			_t = __t112;
			_t = _t.getNextSibling();
		}
		catch (RecognitionException ex) {
			reportError(ex);
			if (_t!=null) {_t = _t.getNextSibling();}
		}
		_retTree = _t;
	}
	
	public final String  decl(AST _t,
		HashMap<String,Object> bindings
	) throws RecognitionException {
		 String r=new String(); ;
		
		AST decl_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		AST id = null;
		Object t;
		
		try {      // for error handling
			AST __t108 = _t;
			id = _t==ASTNULL ? null :(AST)_t;
			match(_t,ID);
			_t = _t.getFirstChild();
			t=type_obj(_t);
			_t = _retTree;
			_t = __t108;
			_t = _t.getNextSibling();
			
			bindings.put(id.getText(), t);
			r = id.getText();
			
		}
		catch (RecognitionException ex) {
			reportError(ex);
			if (_t!=null) {_t = _t.getNextSibling();}
		}
		_retTree = _t;
		return r;
	}
	
	public final Object  type_obj(AST _t) throws RecognitionException {
		Object r=new Object();;
		
		AST type_obj_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		try {      // for error handling
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case INT:
			{
				AST tmp4_AST_in = (AST)_t;
				match(_t,INT);
				_t = _t.getNextSibling();
				r = new Integer(0);
				break;
			}
			case REAL:
			{
				AST tmp5_AST_in = (AST)_t;
				match(_t,REAL);
				_t = _t.getNextSibling();
				r = new Double(0);
				break;
			}
			case BOOL:
			{
				AST tmp6_AST_in = (AST)_t;
				match(_t,BOOL);
				_t = _t.getNextSibling();
				r = new Boolean(false);
				break;
			}
			case PT:
			{
				AST tmp7_AST_in = (AST)_t;
				match(_t,PT);
				_t = _t.getNextSibling();
				r = new Point();
				break;
			}
			case STRING:
			{
				AST tmp8_AST_in = (AST)_t;
				match(_t,STRING);
				_t = _t.getNextSibling();
				r = new String();
				break;
			}
			case NOTHING:
			{
				AST tmp9_AST_in = (AST)_t;
				match(_t,NOTHING);
				_t = _t.getNextSibling();
				r = null;
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			if (_t!=null) {_t = _t.getNextSibling();}
		}
		_retTree = _t;
		return r;
	}
	
	public final Closure.ReturnType  type(AST _t) throws RecognitionException {
		Closure.ReturnType r = Closure.ReturnType.R_NOTHING;;
		
		AST type_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		try {      // for error handling
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case INT:
			{
				AST tmp10_AST_in = (AST)_t;
				match(_t,INT);
				_t = _t.getNextSibling();
				r = Closure.ReturnType.R_INT;
				break;
			}
			case REAL:
			{
				AST tmp11_AST_in = (AST)_t;
				match(_t,REAL);
				_t = _t.getNextSibling();
				r = Closure.ReturnType.R_REAL;
				break;
			}
			case BOOL:
			{
				AST tmp12_AST_in = (AST)_t;
				match(_t,BOOL);
				_t = _t.getNextSibling();
				r = Closure.ReturnType.R_BOOL;
				break;
			}
			case PT:
			{
				AST tmp13_AST_in = (AST)_t;
				match(_t,PT);
				_t = _t.getNextSibling();
				r = Closure.ReturnType.R_PT;
				break;
			}
			case STRING:
			{
				AST tmp14_AST_in = (AST)_t;
				match(_t,STRING);
				_t = _t.getNextSibling();
				r = Closure.ReturnType.R_STR;
				break;
			}
			case NOTHING:
			{
				AST tmp15_AST_in = (AST)_t;
				match(_t,NOTHING);
				_t = _t.getNextSibling();
				r = Closure.ReturnType.R_NOTHING;
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			if (_t!=null) {_t = _t.getNextSibling();}
		}
		_retTree = _t;
		return r;
	}
	
	public final void func(AST _t) throws RecognitionException, InvalidException {
		
		AST func_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		AST id = null;
		AST bl2 = null;
		AST ie = null;
		AST bl = null;
		BlockNode b; Closure.ReturnType returntype; HashMap<String,Object> bindings = new HashMap<String,Object>(); ArrayList<String> pnames = null;
		
		try {      // for error handling
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case ACTN:
			{
				AST __t116 = _t;
				AST tmp16_AST_in = (AST)_t;
				match(_t,ACTN);
				_t = _t.getFirstChild();
				id = (AST)_t;
				match(_t,ID);
				_t = _t.getNextSibling();
				bl2 = (AST)_t;
				if ( _t==null ) throw new MismatchedTokenException();
				_t = _t.getNextSibling();
				_t = __t116;
				_t = _t.getNextSibling();
				
				statementnumber = 0;
				try
				{
				b=block(bl2,bindings,Closure.ReturnType.R_NOTHING);
				}
				catch (InvalidException e)
				{
				throw new InvalidException("Error in action: " + id.getText() + "\n\tAt statement: " + statementnumber + "\n\t" + e);
				}
				globals.put(id.getText(),new Closure(b,Closure.ReturnType.R_NOTHING,new ArrayList<String>(),new HashMap<String,Object>())); 
				
				break;
			}
			case FUNC:
			{
				AST __t117 = _t;
				AST tmp17_AST_in = (AST)_t;
				match(_t,FUNC);
				_t = _t.getFirstChild();
				ie = (AST)_t;
				match(_t,ID);
				_t = _t.getNextSibling();
				{
				if (_t==null) _t=ASTNULL;
				switch ( _t.getType()) {
				case DECLS:
				{
					pnames=decls(_t,bindings);
					_t = _retTree;
					break;
				}
				case INT:
				case BOOL:
				case REAL:
				case STRING:
				case PT:
				case NOTHING:
				{
					break;
				}
				default:
				{
					throw new NoViableAltException(_t);
				}
				}
				}
				returntype=type(_t);
				_t = _retTree;
				bl = (AST)_t;
				if ( _t==null ) throw new MismatchedTokenException();
				_t = _t.getNextSibling();
				_t = __t117;
				_t = _t.getNextSibling();
				
				if (pnames == null)
				{
				pnames = new ArrayList<String>();
				}
				
				statementnumber = 0;
				try
				{
				b=block(bl,bindings,returntype);
				
				if (b.getReturnType() != returntype)
				{
				throw new InvalidException("function returns " + Closure.getReturnTypeString(b.getReturnType()) + ", declared to return: " + Closure.getReturnTypeString(returntype));
				}
				}
				catch (InvalidException e)
				{
				throw new InvalidException("Error in function: " + ie.getText() + "\n\tAt statement: " + statementnumber + "\n\t" + e);
				}
				
				globals.put(ie.getText(),new Closure(b,returntype,pnames,bindings));
				
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			if (_t!=null) {_t = _t.getNextSibling();}
		}
		_retTree = _t;
	}
	
	public final BlockNode  block(AST _t,
		HashMap<String,Object> bindings, Closure.ReturnType returntype
	) throws RecognitionException, InvalidException {
		BlockNode r=null;;
		
		AST block_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		ArrayList<BlockNode> b;
		
		try {      // for error handling
			AST __t120 = _t;
			AST tmp18_AST_in = (AST)_t;
			match(_t,BLOCK);
			_t = _t.getFirstChild();
			b=block_innards(_t,bindings,returntype);
			_t = _retTree;
			_t = __t120;
			_t = _t.getNextSibling();
			
			r = new BlockNode(BlockNode.Operation.S_BLOCK,null,b,scanReturnType(b,returntype));
			
		}
		catch (RecognitionException ex) {
			reportError(ex);
			if (_t!=null) {_t = _t.getNextSibling();}
		}
		_retTree = _t;
		return r;
	}
	
	public final ArrayList<BlockNode>  block_innards(AST _t,
		HashMap<String,Object> bindings, Closure.ReturnType returntype
	) throws RecognitionException, InvalidException {
		ArrayList<BlockNode> r=new ArrayList<BlockNode>();;
		
		AST block_innards_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		BlockNode s;
		
		try {      // for error handling
			{
			_loop124:
			do {
				if (_t==null) _t=ASTNULL;
				if ((_tokenSet_0.member(_t.getType()))) {
					{
					s=statement(_t,bindings,returntype);
					_t = _retTree;
					}
					
					if (s != null)
					{
					r.add(s);
					}
					
				}
				else {
					break _loop124;
				}
				
			} while (true);
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			if (_t!=null) {_t = _t.getNextSibling();}
		}
		_retTree = _t;
		return r;
	}
	
	public final BlockNode  statement(AST _t,
		HashMap<String,Object> bindings, Closure.ReturnType returntype
	) throws RecognitionException, InvalidException {
		BlockNode r=null;;
		
		AST statement_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		AST id = null;
		AST rstatement = null;
		Object t; BlockNode e,b,b2 = null; ArrayList<BlockNode> el = null;
		
		try {      // for error handling
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case ID:
			{
				AST __t126 = _t;
				id = _t==ASTNULL ? null :(AST)_t;
				match(_t,ID);
				_t = _t.getFirstChild();
				t=type_obj(_t);
				_t = _retTree;
				_t = __t126;
				_t = _t.getNextSibling();
				
				statementnumber += 1;
				if (bindings.containsKey(id.getText()))
				{
				throw new InvalidException("variable '" + id.getText() + "' redeclared");
				}
				bindings.put(id.getText(),t);
				
				break;
			}
			case IF:
			{
				AST __t127 = _t;
				AST tmp19_AST_in = (AST)_t;
				match(_t,IF);
				_t = _t.getFirstChild();
				e=boolop(_t,bindings);
				_t = _retTree;
				b=block(_t,bindings,returntype);
				_t = _retTree;
				{
				if (_t==null) _t=ASTNULL;
				switch ( _t.getType()) {
				case ELIFS:
				{
					el=elseifs(_t,bindings,returntype);
					_t = _retTree;
					break;
				}
				case 3:
				case BLOCK:
				{
					break;
				}
				default:
				{
					throw new NoViableAltException(_t);
				}
				}
				}
				{
				if (_t==null) _t=ASTNULL;
				switch ( _t.getType()) {
				case BLOCK:
				{
					b2=block(_t,bindings,returntype);
					_t = _retTree;
					break;
				}
				case 3:
				{
					break;
				}
				default:
				{
					throw new NoViableAltException(_t);
				}
				}
				}
				_t = __t127;
				_t = _t.getNextSibling();
				
				ArrayList<BlockNode> args = new ArrayList<BlockNode>();
				
				if (e.getReturnType() != Closure.ReturnType.R_BOOL)
				{
				throw new InvalidException("if statement requires boolean check");
				}
				
				args.add(e);
				args.add(b);
				
				boolean returns = (b.getReturnType() == returntype);
				
				if (el != null)
				{
				for (int i = 0; i < el.size(); i += 2)
				{
				// These come in pairs - expression, then block
				BlockNode be = el.get(i);
				args.add(be);
				
				BlockNode bb = el.get(i+1);
				args.add(bb);
				
				returns = returns && (bb.getReturnType() == returntype);
				}
				}
				
				if (b2 != null)
				{
				args.add(b2);
				
				returns = returns && (b2.getReturnType() == returntype);
				}
				
				if (returns)
				{
				r = new BlockNode(BlockNode.Operation.S_IF,null,args,returntype);
				}
				else
				{
				r = new BlockNode(BlockNode.Operation.S_IF,null,args,Closure.ReturnType.R_NOTHING);
				}
				
				break;
			}
			case WHILE:
			{
				AST __t130 = _t;
				AST tmp20_AST_in = (AST)_t;
				match(_t,WHILE);
				_t = _t.getFirstChild();
				e=boolop(_t,bindings);
				_t = _retTree;
				b=block(_t,bindings,returntype);
				_t = _retTree;
				_t = __t130;
				_t = _t.getNextSibling();
				
				ArrayList<BlockNode> args = new ArrayList<BlockNode>();
				
				if (e.getReturnType() != Closure.ReturnType.R_BOOL)
				{
				throw new InvalidException("while statement requires boolean check");
				}
				
				args.add(e);
				args.add(b);
				
				if (b.getReturnType() == returntype)
				{
				r = new BlockNode(BlockNode.Operation.S_WHILE,null,args,returntype);
				}
				else if (b.getReturnType() == Closure.ReturnType.R_NOTHING)
				{
				r = new BlockNode(BlockNode.Operation.S_WHILE,null,args,Closure.ReturnType.R_NOTHING);
				}
				else
				{
				throw new InvalidException("function returns " + Closure.getReturnTypeString(b.getReturnType()) + " , declared to return: " + Closure.getReturnTypeString(returntype));
				}
				
				break;
			}
			case CALL:
			{
				r=call(_t,bindings,true);
				_t = _retTree;
				break;
			}
			case ASSIGN:
			case ASSIGN_DOTX:
			case ASSIGN_DOTY:
			{
				r=assign(_t,bindings);
				_t = _retTree;
				break;
			}
			case RETURN:
			{
				AST __t131 = _t;
				AST tmp21_AST_in = (AST)_t;
				match(_t,RETURN);
				_t = _t.getFirstChild();
				{
				if (_t==null) _t=ASTNULL;
				switch ( _t.getType()) {
				case FUNC:
				case ACTN:
				case TAKES:
				case RETURNS:
				case IF:
				case ELSEIF:
				case ELSE:
				case WHILE:
				case RETURN:
				case INT:
				case BOOL:
				case REAL:
				case STRING:
				case PT:
				case FALSE:
				case TRUE:
				case NOTHING:
				case LPAREN:
				case RPAREN:
				case LBRACE:
				case RBRACE:
				case LBRACKET:
				case RBRACKET:
				case SEMI:
				case COLON:
				case COMMA:
				case EQUAL:
				case EQ:
				case NE:
				case AND:
				case OR:
				case LT:
				case LE:
				case GT:
				case GE:
				case ADD:
				case SUB:
				case MULT:
				case DIV:
				case BANG:
				case DOTX:
				case DOTY:
				case ID:
				case NUMBER:
				case STR:
				case WS:
				case COMMENT:
				case SCRIPT:
				case DECLS:
				case FUNCS:
				case BLOCK:
				case CALL:
				case ASSIGN:
				case ASSIGN_DOTX:
				case ASSIGN_DOTY:
				case NEG:
				case POINT:
				case ELIFS:
				{
					rstatement = (AST)_t;
					if ( _t==null ) throw new MismatchedTokenException();
					_t = _t.getNextSibling();
					break;
				}
				case 3:
				{
					break;
				}
				default:
				{
					throw new NoViableAltException(_t);
				}
				}
				}
				_t = __t131;
				_t = _t.getNextSibling();
				
				statementnumber += 1;
				ArrayList<BlockNode> args = new ArrayList<BlockNode>();
				
				if (rstatement != null)
				{
				e = boolop(rstatement, bindings);
				if (e.getReturnType() != returntype)
				{
				throw new InvalidException("function returns " + Closure.getReturnTypeString(e.getReturnType()) + ", declared to return: " + Closure.getReturnTypeString(returntype));
				}
				args.add(e);
				}
				else if (returntype != Closure.ReturnType.R_NOTHING)
				{
				throw new InvalidException("function returns nothing, declared to return: " + Closure.getReturnTypeString(returntype));
				}
				
				r = new BlockNode(BlockNode.Operation.S_RETURN,null,args,returntype);
				
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			if (_t!=null) {_t = _t.getNextSibling();}
		}
		_retTree = _t;
		return r;
	}
	
	public final BlockNode  boolop(AST _t,
		HashMap<String,Object> bindings
	) throws RecognitionException, InvalidException {
		BlockNode r=null;;
		
		AST boolop_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		BlockNode a,b;
		
		try {      // for error handling
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case AND:
			{
				AST __t150 = _t;
				AST tmp22_AST_in = (AST)_t;
				match(_t,AND);
				_t = _t.getFirstChild();
				a=boolop(_t,bindings);
				_t = _retTree;
				b=boolop(_t,bindings);
				_t = _retTree;
				_t = __t150;
				_t = _t.getNextSibling();
				r = doBoolop(a,b,BlockNode.Operation.S_AND);
				break;
			}
			case OR:
			{
				AST __t151 = _t;
				AST tmp23_AST_in = (AST)_t;
				match(_t,OR);
				_t = _t.getFirstChild();
				a=boolop(_t,bindings);
				_t = _retTree;
				b=boolop(_t,bindings);
				_t = _retTree;
				_t = __t151;
				_t = _t.getNextSibling();
				r = doBoolop(a,b,BlockNode.Operation.S_OR);
				break;
			}
			case EQ:
			{
				AST __t152 = _t;
				AST tmp24_AST_in = (AST)_t;
				match(_t,EQ);
				_t = _t.getFirstChild();
				a=intop(_t,bindings);
				_t = _retTree;
				b=intop(_t,bindings);
				_t = _retTree;
				_t = __t152;
				_t = _t.getNextSibling();
				r = doCompare(a,b,BlockNode.Operation.S_EQ);
				break;
			}
			case NE:
			{
				AST __t153 = _t;
				AST tmp25_AST_in = (AST)_t;
				match(_t,NE);
				_t = _t.getFirstChild();
				a=intop(_t,bindings);
				_t = _retTree;
				b=intop(_t,bindings);
				_t = _retTree;
				_t = __t153;
				_t = _t.getNextSibling();
				r = doCompare(a,b,BlockNode.Operation.S_NE);
				break;
			}
			case LT:
			{
				AST __t154 = _t;
				AST tmp26_AST_in = (AST)_t;
				match(_t,LT);
				_t = _t.getFirstChild();
				a=intop(_t,bindings);
				_t = _retTree;
				b=intop(_t,bindings);
				_t = _retTree;
				_t = __t154;
				_t = _t.getNextSibling();
				r = doCompare(a,b,BlockNode.Operation.S_LT);
				break;
			}
			case GT:
			{
				AST __t155 = _t;
				AST tmp27_AST_in = (AST)_t;
				match(_t,GT);
				_t = _t.getFirstChild();
				a=intop(_t,bindings);
				_t = _retTree;
				b=intop(_t,bindings);
				_t = _retTree;
				_t = __t155;
				_t = _t.getNextSibling();
				r = doCompare(a,b,BlockNode.Operation.S_GT);
				break;
			}
			case LE:
			{
				AST __t156 = _t;
				AST tmp28_AST_in = (AST)_t;
				match(_t,LE);
				_t = _t.getFirstChild();
				a=intop(_t,bindings);
				_t = _retTree;
				b=intop(_t,bindings);
				_t = _retTree;
				_t = __t156;
				_t = _t.getNextSibling();
				r = doCompare(a,b,BlockNode.Operation.S_LE);
				break;
			}
			case GE:
			{
				AST __t157 = _t;
				AST tmp29_AST_in = (AST)_t;
				match(_t,GE);
				_t = _t.getFirstChild();
				a=intop(_t,bindings);
				_t = _retTree;
				b=intop(_t,bindings);
				_t = _retTree;
				_t = __t157;
				_t = _t.getNextSibling();
				r = doCompare(a,b,BlockNode.Operation.S_GE);
				break;
			}
			case BANG:
			{
				AST __t158 = _t;
				AST tmp30_AST_in = (AST)_t;
				match(_t,BANG);
				_t = _t.getFirstChild();
				a=boolop(_t,bindings);
				_t = _retTree;
				_t = __t158;
				_t = _t.getNextSibling();
				
				if (a.getReturnType() != Closure.ReturnType.R_BOOL)
				{
				throw new InvalidException("! operator only negates Boolean");
				}
				ArrayList<BlockNode> args = new ArrayList<BlockNode>();
				args.add(a);
				r = new BlockNode(BlockNode.Operation.S_BANG,null,args,Closure.ReturnType.R_BOOL);
				
				break;
			}
			case TRUE:
			{
				AST tmp31_AST_in = (AST)_t;
				match(_t,TRUE);
				_t = _t.getNextSibling();
				r = new BlockNode(new Boolean(true));
				break;
			}
			case FALSE:
			{
				AST tmp32_AST_in = (AST)_t;
				match(_t,FALSE);
				_t = _t.getNextSibling();
				r = new BlockNode(new Boolean(false));
				break;
			}
			case ADD:
			case SUB:
			case MULT:
			case DIV:
			case DOTX:
			case DOTY:
			case ID:
			case NUMBER:
			case STR:
			case CALL:
			case NEG:
			case POINT:
			{
				r=intop(_t,bindings);
				_t = _retTree;
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			if (_t!=null) {_t = _t.getNextSibling();}
		}
		_retTree = _t;
		return r;
	}
	
	public final ArrayList<BlockNode>  elseifs(AST _t,
		HashMap<String,Object> bindings, Closure.ReturnType returntype
	) throws RecognitionException, InvalidException {
		ArrayList<BlockNode> r = new ArrayList<BlockNode>();;
		
		AST elseifs_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		
		try {      // for error handling
			AST __t140 = _t;
			AST tmp33_AST_in = (AST)_t;
			match(_t,ELIFS);
			_t = _t.getFirstChild();
			{
			int _cnt142=0;
			_loop142:
			do {
				if (_t==null) _t=ASTNULL;
				if ((_t.getType()==ELSEIF)) {
					elseif(_t,bindings,returntype,r);
					_t = _retTree;
				}
				else {
					if ( _cnt142>=1 ) { break _loop142; } else {throw new NoViableAltException(_t);}
				}
				
				_cnt142++;
			} while (true);
			}
			_t = __t140;
			_t = _t.getNextSibling();
		}
		catch (RecognitionException ex) {
			reportError(ex);
			if (_t!=null) {_t = _t.getNextSibling();}
		}
		_retTree = _t;
		return r;
	}
	
	public final BlockNode  call(AST _t,
		HashMap<String,Object> bindings,boolean individual
	) throws RecognitionException, InvalidException {
		BlockNode r=null;;
		
		AST call_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		AST id = null;
		ArrayList<BlockNode> e;
		
		try {      // for error handling
			AST __t138 = _t;
			AST tmp34_AST_in = (AST)_t;
			match(_t,CALL);
			_t = _t.getFirstChild();
			id = (AST)_t;
			match(_t,ID);
			_t = _t.getNextSibling();
			e=exprs(_t,bindings);
			_t = _retTree;
			_t = __t138;
			_t = _t.getNextSibling();
			
			if (individual)
			{
			statementnumber += 1;
			}
			if (!globals.containsKey(id.getText()))
			{
			throw new InvalidException("function called without being defined: " + id.getText());
			}
			
			Closure func = (Closure)globals.get(id.getText());
			ArrayList<Closure.ReturnType> types = func.getParamTypes();
			Closure.ReturnType returntype = func.getReturnType();
			
			if (e.size() != types.size())
			{
			throw new InvalidException("function " + id.getText() + " called with " + e.size() + " parameters; expects " + types.size());
			}
			
			for (int i = 0; i < e.size(); i++)
			{
			BlockNode b = e.get(i);
			if (b.getReturnType() != types.get(i))
			{
			throw new InvalidException("function " + id.getText() + " called with mismatched parameter type(s)");
			}
			}
			
			e.add(0, new BlockNode(func));
			
			r = new BlockNode(BlockNode.Operation.S_CALL,null,e,returntype);
			
		}
		catch (RecognitionException ex) {
			reportError(ex);
			if (_t!=null) {_t = _t.getNextSibling();}
		}
		_retTree = _t;
		return r;
	}
	
	public final BlockNode  assign(AST _t,
		HashMap<String,Object> bindings
	) throws RecognitionException, InvalidException {
		BlockNode r=null;;
		
		AST assign_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		AST id = null;
		AST ia = null;
		AST ib = null;
		BlockNode b;
		
		try {      // for error handling
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case ASSIGN:
			{
				AST __t134 = _t;
				AST tmp35_AST_in = (AST)_t;
				match(_t,ASSIGN);
				_t = _t.getFirstChild();
				id = (AST)_t;
				match(_t,ID);
				_t = _t.getNextSibling();
				b=boolop(_t,bindings);
				_t = _retTree;
				_t = __t134;
				_t = _t.getNextSibling();
				
				statementnumber += 1;
				Closure.ReturnType type = b.getReturnType();
				String i = id.getText();
				
				ArrayList<BlockNode> args = new ArrayList<BlockNode>();
				args.add(new BlockNode(BlockNode.Operation.S_ID,i,null,typeCheck(i,type,bindings)));
				args.add(b);
				r = new BlockNode(BlockNode.Operation.S_ASSIGN,null,args,Closure.ReturnType.R_NOTHING);
				
				break;
			}
			case ASSIGN_DOTX:
			{
				AST __t135 = _t;
				AST tmp36_AST_in = (AST)_t;
				match(_t,ASSIGN_DOTX);
				_t = _t.getFirstChild();
				ia = (AST)_t;
				match(_t,ID);
				_t = _t.getNextSibling();
				b=boolop(_t,bindings);
				_t = _retTree;
				_t = __t135;
				_t = _t.getNextSibling();
				
				statementnumber += 1;
				Closure.ReturnType type = b.getReturnType();
				
				if (type != Closure.ReturnType.R_REAL &&
				type != Closure.ReturnType.R_INT)
				{
				throw new InvalidException("Type mismatch, expected: real found: " + Closure.getReturnTypeString(type));
				}
				
				String i = ia.getText();
				
				ArrayList<BlockNode> args = new ArrayList<BlockNode>();
				args.add(new BlockNode(BlockNode.Operation.S_ID,i,null,Closure.ReturnType.R_PT));
				args.add(b);
				r = new BlockNode(BlockNode.Operation.S_ASSIGN_DOTX,null,args,Closure.ReturnType.R_NOTHING);
				
				break;
			}
			case ASSIGN_DOTY:
			{
				AST __t136 = _t;
				AST tmp37_AST_in = (AST)_t;
				match(_t,ASSIGN_DOTY);
				_t = _t.getFirstChild();
				ib = (AST)_t;
				match(_t,ID);
				_t = _t.getNextSibling();
				b=boolop(_t,bindings);
				_t = _retTree;
				_t = __t136;
				_t = _t.getNextSibling();
				
				statementnumber += 1;
				Closure.ReturnType type = b.getReturnType();
				
				if (type != Closure.ReturnType.R_REAL &&
				type != Closure.ReturnType.R_INT)
				{
				throw new InvalidException("Type mismatch, expected: real found: " + Closure.getReturnTypeString(type));
				}
				
				String i = ib.getText();
				
				ArrayList<BlockNode> args = new ArrayList<BlockNode>();
				args.add(new BlockNode(BlockNode.Operation.S_ID,i,null,Closure.ReturnType.R_PT));
				args.add(b);
				r = new BlockNode(BlockNode.Operation.S_ASSIGN_DOTY,null,args,Closure.ReturnType.R_NOTHING);
				
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			if (_t!=null) {_t = _t.getNextSibling();}
		}
		_retTree = _t;
		return r;
	}
	
	public final ArrayList<BlockNode>  exprs(AST _t,
		HashMap<String,Object> bindings
	) throws RecognitionException, InvalidException {
		ArrayList<BlockNode> r=new ArrayList<BlockNode>();;
		
		AST exprs_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		BlockNode e;
		
		try {      // for error handling
			{
			_loop148:
			do {
				if (_t==null) _t=ASTNULL;
				if ((_tokenSet_1.member(_t.getType()))) {
					{
					e=boolop(_t,bindings);
					_t = _retTree;
					}
					r.add(e);
				}
				else {
					break _loop148;
				}
				
			} while (true);
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			if (_t!=null) {_t = _t.getNextSibling();}
		}
		_retTree = _t;
		return r;
	}
	
	public final void elseif(AST _t,
		HashMap<String,Object> bindings, Closure.ReturnType returntype, ArrayList<BlockNode> nodes
	) throws RecognitionException, InvalidException {
		
		AST elseif_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		BlockNode c,b;
		
		try {      // for error handling
			AST __t144 = _t;
			AST tmp38_AST_in = (AST)_t;
			match(_t,ELSEIF);
			_t = _t.getFirstChild();
			c=boolop(_t,bindings);
			_t = _retTree;
			b=block(_t,bindings,returntype);
			_t = _retTree;
			_t = __t144;
			_t = _t.getNextSibling();
			
			if (c.getReturnType() != Closure.ReturnType.R_BOOL)
			{
			throw new InvalidException("elseif statement requires boolean check");
			}
			nodes.add(c);
			nodes.add(b);
			
		}
		catch (RecognitionException ex) {
			reportError(ex);
			if (_t!=null) {_t = _t.getNextSibling();}
		}
		_retTree = _t;
	}
	
	public final BlockNode  intop(AST _t,
		HashMap<String,Object> bindings
	) throws RecognitionException, InvalidException {
		BlockNode r=null;;
		
		AST intop_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		AST n = null;
		BlockNode a,b;
		
		try {      // for error handling
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case ADD:
			{
				AST __t160 = _t;
				AST tmp39_AST_in = (AST)_t;
				match(_t,ADD);
				_t = _t.getFirstChild();
				a=intop(_t,bindings);
				_t = _retTree;
				b=intop(_t,bindings);
				_t = _retTree;
				_t = __t160;
				_t = _t.getNextSibling();
				r = doIntop(a,b,BlockNode.Operation.S_ADD);
				break;
			}
			case SUB:
			{
				AST __t161 = _t;
				AST tmp40_AST_in = (AST)_t;
				match(_t,SUB);
				_t = _t.getFirstChild();
				a=intop(_t,bindings);
				_t = _retTree;
				b=intop(_t,bindings);
				_t = _retTree;
				_t = __t161;
				_t = _t.getNextSibling();
				r = doIntop(a,b,BlockNode.Operation.S_SUB);
				break;
			}
			case MULT:
			{
				AST __t162 = _t;
				AST tmp41_AST_in = (AST)_t;
				match(_t,MULT);
				_t = _t.getFirstChild();
				a=intop(_t,bindings);
				_t = _retTree;
				b=intop(_t,bindings);
				_t = _retTree;
				_t = __t162;
				_t = _t.getNextSibling();
				r = doIntop(a,b,BlockNode.Operation.S_MULT);
				break;
			}
			case DIV:
			{
				AST __t163 = _t;
				AST tmp42_AST_in = (AST)_t;
				match(_t,DIV);
				_t = _t.getFirstChild();
				a=intop(_t,bindings);
				_t = _retTree;
				b=intop(_t,bindings);
				_t = _retTree;
				_t = __t163;
				_t = _t.getNextSibling();
				r = doIntop(a,b,BlockNode.Operation.S_DIV);
				break;
			}
			case NEG:
			{
				AST __t164 = _t;
				AST tmp43_AST_in = (AST)_t;
				match(_t,NEG);
				_t = _t.getFirstChild();
				a=intop(_t,bindings);
				_t = _retTree;
				_t = __t164;
				_t = _t.getNextSibling();
				
				Closure.ReturnType aret = a.getReturnType();
				if (aret != Closure.ReturnType.R_INT &&
				aret != Closure.ReturnType.R_REAL  &&
				aret != Closure.ReturnType.R_PT)
				{
				throw new InvalidException("- operator only negates Int, Real, and Point");
				}
				ArrayList<BlockNode> args = new ArrayList<BlockNode>();
				args.add(a);
				r = new BlockNode(BlockNode.Operation.S_NEG,null,args,aret);
				
				break;
			}
			case NUMBER:
			{
				n = (AST)_t;
				match(_t,NUMBER);
				_t = _t.getNextSibling();
				
				String num = n.getText();
				if (num.contains("."))
				{
				r = new BlockNode(new Double(num));
				}
				else
				{
				r = new BlockNode(new Integer(num));
				}
				
				break;
			}
			case CALL:
			{
				r=call(_t,bindings,false);
				_t = _retTree;
				break;
			}
			case DOTX:
			case DOTY:
			case ID:
			case STR:
			case POINT:
			{
				r=item(_t,bindings);
				_t = _retTree;
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			if (_t!=null) {_t = _t.getNextSibling();}
		}
		_retTree = _t;
		return r;
	}
	
	public final BlockNode  item(AST _t,
		HashMap<String,Object> bindings
	) throws RecognitionException, InvalidException {
		BlockNode r=null;;
		
		AST item_AST_in = (_t == ASTNULL) ? null : (AST)_t;
		AST i = null;
		AST ix = null;
		AST iy = null;
		AST s = null;
		BlockNode a,b;
		
		try {      // for error handling
			if (_t==null) _t=ASTNULL;
			switch ( _t.getType()) {
			case ID:
			{
				i = (AST)_t;
				match(_t,ID);
				_t = _t.getNextSibling();
				
				String id = i.getText();
				if (bindings.containsKey(id))
				{
				r = new BlockNode(BlockNode.Operation.S_ID,id,null,Closure.getReturnType(bindings.get(id)));
				}
				else if (globals.containsKey(id))
				{
				r = new BlockNode(BlockNode.Operation.S_ID,id,null,Closure.getReturnType(globals.get(id)));
				}
				else
				{
				throw new InvalidException("variable used without being declared: " + id);
				}
				
				break;
			}
			case POINT:
			{
				AST __t166 = _t;
				AST tmp44_AST_in = (AST)_t;
				match(_t,POINT);
				_t = _t.getFirstChild();
				a=intop(_t,bindings);
				_t = _retTree;
				b=intop(_t,bindings);
				_t = _retTree;
				_t = __t166;
				_t = _t.getNextSibling();
				
				Closure.ReturnType aret = a.getReturnType();
				Closure.ReturnType bret = b.getReturnType();
				if ((aret != Closure.ReturnType.R_INT &&
				aret != Closure.ReturnType.R_REAL) ||
				(bret != Closure.ReturnType.R_INT &&
				bret != Closure.ReturnType.R_REAL))
				{
				throw new InvalidException("Point may only contain Int or Real");
				}
				
				ArrayList<BlockNode> args = new ArrayList<BlockNode>();
				args.add(a);
				args.add(b);
				
				r = new BlockNode(BlockNode.Operation.S_POINT,null,args,Closure.ReturnType.R_PT);
				
				break;
			}
			case DOTX:
			{
				AST __t167 = _t;
				AST tmp45_AST_in = (AST)_t;
				match(_t,DOTX);
				_t = _t.getFirstChild();
				ix = (AST)_t;
				match(_t,ID);
				_t = _t.getNextSibling();
				_t = __t167;
				_t = _t.getNextSibling();
				
				String id = ix.getText();
				if (bindings.containsKey(id))
				{
				if (Closure.getReturnType(bindings.get(id)) == Closure.ReturnType.R_PT)
				{
				r = new BlockNode(BlockNode.Operation.S_PTX,id,null,Closure.ReturnType.R_REAL);
				}
				else
				{
				throw new InvalidException("dot operator expects type Point");
				}
				}
				else if (globals.containsKey(id))
				{
				if (Closure.getReturnType(globals.get(id)) == Closure.ReturnType.R_PT)
				{
				r = new BlockNode(BlockNode.Operation.S_PTX,id,null,Closure.ReturnType.R_REAL);
				}
				else
				{
				throw new InvalidException("dot operator expects type Point");
				}
				}
				else
				{
				throw new InvalidException("variable used without being declared: " + id);
				}
				
				break;
			}
			case DOTY:
			{
				AST __t168 = _t;
				AST tmp46_AST_in = (AST)_t;
				match(_t,DOTY);
				_t = _t.getFirstChild();
				iy = (AST)_t;
				match(_t,ID);
				_t = _t.getNextSibling();
				_t = __t168;
				_t = _t.getNextSibling();
				
				String id = iy.getText();
				if (bindings.containsKey(id))
				{
				if (Closure.getReturnType(bindings.get(id)) == Closure.ReturnType.R_PT)
				{
				r = new BlockNode(BlockNode.Operation.S_PTY,id,null,Closure.ReturnType.R_REAL);
				}
				else
				{
				throw new InvalidException("dot operator expects type Point");
				}
				}
				else if (globals.containsKey(id))
				{
				if (Closure.getReturnType(globals.get(id)) == Closure.ReturnType.R_PT)
				{
				r = new BlockNode(BlockNode.Operation.S_PTY,id,null,Closure.ReturnType.R_REAL);
				}
				else
				{
				throw new InvalidException("dot operator expects type Point");
				}
				}
				else
				{
				throw new InvalidException("variable used without being declared: " + id);
				}
				
				break;
			}
			case STR:
			{
				s = (AST)_t;
				match(_t,STR);
				_t = _t.getNextSibling();
				
				String t_s = s.getText();
				r = new BlockNode(t_s.substring(1,t_s.length()-1).replace("\\n","\n"));
				
				break;
			}
			default:
			{
				throw new NoViableAltException(_t);
			}
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			if (_t!=null) {_t = _t.getNextSibling();}
		}
		_retTree = _t;
		return r;
	}
	
	
	public static final String[] _tokenNames = {
		"<0>",
		"EOF",
		"<2>",
		"NULL_TREE_LOOKAHEAD",
		"\"function\"",
		"\"action\"",
		"\"takes\"",
		"\"returns\"",
		"\"if\"",
		"\"elseif\"",
		"\"else\"",
		"\"while\"",
		"\"return\"",
		"\"int\"",
		"\"bool\"",
		"\"real\"",
		"\"string\"",
		"\"point\"",
		"\"false\"",
		"\"true\"",
		"\"nothing\"",
		"LPAREN",
		"RPAREN",
		"LBRACE",
		"RBRACE",
		"LBRACKET",
		"RBRACKET",
		"SEMI",
		"COLON",
		"COMMA",
		"EQUAL",
		"EQ",
		"NE",
		"AND",
		"OR",
		"LT",
		"LE",
		"GT",
		"GE",
		"ADD",
		"SUB",
		"MULT",
		"DIV",
		"BANG",
		"DOTX",
		"DOTY",
		"ID",
		"NUMBER",
		"STR",
		"WS",
		"COMMENT",
		"SCRIPT",
		"DECLS",
		"FUNCS",
		"BLOCK",
		"CALL",
		"ASSIGN",
		"ASSIGN_DOTX",
		"ASSIGN_DOTY",
		"NEG",
		"POINT",
		"ELIFS"
	};
	
	private static final long[] mk_tokenSet_0() {
		long[] data = { 540502324028643584L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_0 = new BitSet(mk_tokenSet_0());
	private static final long[] mk_tokenSet_1() {
		long[] data = { 1765974001735958528L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_1 = new BitSet(mk_tokenSet_1());
	}
	
