package cage.graphics;

import org.minueto.image.MinuetoImageFile;
import org.minueto.image.MinuetoImage;
import org.minueto.MinuetoFileException;
import cage.core.ResourceEngine;
import cage.core.Logger;

import java.net.URL;

/** 
 * Image is a wrapper class for the image datatype used by the Cage
 * backend. All Cage classes should use this as their Image data type
 * since it automatically registers itself with the ResourceEngine
 * when created and allows a separation of Cage code and the
 * underlying engine framework.
 * 
 * @author Seth Marinello
 * @date May 24, 2007
 */
public class Image implements VisualItem {

    /** The name. */
    private String name;

    /** The file name. */
    private String fileName;

    /** The image. */
    private MinuetoImage image;

    /**
     * Instantiates a new image.
     * 
     * @param name the name
     * @param fileName the file name
     */
    public Image(String name, String fileName) {

        this.name = name;
        this.fileName = fileName;

        URL imageFile = ResourceEngine.locateResource(fileName);

        if (imageFile == null) {
            Logger.getRootLogger().error(fileName + " could not be loaded successfully");
        }
        else {

            try {
                this.image = new MinuetoImageFile(imageFile);
            } catch (MinuetoFileException ex) {
                Logger.getRootLogger().error("Could not find file " + fileName);
            }

            ResourceEngine.registerVisualItem(name,this);
        }
    }

    /* (non-Javadoc)
     * @see cage.graphics.VisualItem#scale(double, double)
     */
    public void scale(double xScale, double yScale) {
        image = image.scale(xScale, yScale);
    }

    /* (non-Javadoc)
     * @see cage.graphics.VisualItem#getName()
     */
    public String getName() {
        return name;
    }

    /* (non-Javadoc)
     * @see cage.graphics.VisualItem#getImage()
     */
    public MinuetoImage getImage() {
        return image;
    }

    /* (non-Javadoc)
     * @see cage.graphics.VisualItem#getWidth()
     */
    public int getWidth() {
        return image.getWidth();
    }

    /* (non-Javadoc)
     * @see cage.graphics.VisualItem#getHeight()
     */
    public int getHeight() {
        return image.getHeight();
    }

    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return  "(Image) " + name + " file:" + fileName;
    }
}
