package cage.input;

/**
 * Represents a key value and action name pair. The values for keyCode are taken
 * from MinuetoKeyboard. When defining a binding in either a Scene or a Production
 * XML element, the following strings are valid names of a key value. Case matters.
 * <ul>
 *<li>KEY_A
 *<li>KEY_B
 *<li>KEY_C
 *<li>KEY_D
 *<li>KEY_E
 *<li>KEY_F
 *<li>KEY_G
 *<li>KEY_H
 *<li>KEY_I
 *<li>KEY_J
 *<li>KEY_K
 *<li>KEY_L
 *<li>KEY_M
 *<li>KEY_N
 *<li>KEY_O
 *<li>KEY_P
 *<li>KEY_Q
 *<li>KEY_R
 *<li>KEY_S
 *<li>KEY_T
 *<li>KEY_U
 *<li>KEY_V
 *<li>KEY_W
 *<li>KEY_X
 *<li>KEY_Y
 *<li>KEY_Z
 *<li>KEY_1
 *<li>KEY_2
 *<li>KEY_3
 *<li>KEY_4
 *<li>KEY_5
 *<li>KEY_6
 *<li>KEY_7
 *<li>KEY_8
 *<li>KEY_9
 *<li>KEY_0
 *<li>KEY_CAPSLOCK
 *<li>KEY_SHIFT
 *<li>KEY_CONTROL
 *<li>KEY_ALT
 *<li>KEY_ENTER
 *<li>KEY_BACKSPACE
 *<li>KEY_SPACE
 *<li>KEY_UP
 *<li>KEY_DOWN
 *<li>KEY_LEFT
 *<li>KEY_RIGHT
 *<li>KEY_DELETE
 *<li>KEY_ESC
 *<li>KEY_NUM_1
 *<li>KEY_NUM_2
 *<li>KEY_NUM_3
 *<li>KEY_NUM_4
 *<li>KEY_NUM_5
 *<li>KEY_NUM_6
 *<li>KEY_NUM_7
 *<li>KEY_NUM_8
 *<li>KEY_NUM_9
 *<li>KEY_NUM_0
 *<li>KEY_F1
 *<li>KEY_F2
 *<li>KEY_F3
 *<li>KEY_F4
 *<li>KEY_F5
 *<li>KEY_F6
 *<li>KEY_F7
 *<li>KEY_F8
 *<li>KEY_F9
 *<li>KEY_F10
 *<li>KEY_F11
 *<li>KEY_F12
 *<li>KEY_PAGEUP
 *<li>KEY_PAGEDOWN
 *<li>KEY_END
 *<li>KEY_HOME
 * </ul>
 *
 * @author Seth Marinello
 */
public class Binding {

    /** The name of the action to call when this binding is invoked */
    public String actionName;

    /**
     * The key value to bind the action name to. Used by Cage.input.KeyboardHandler
     * It is initialized to -1 because that is not a valid keyCode and thus will be
     * ignored by KeyboardHandler if a binding is added without being initialized.
     */
    public int keyCode = -1;

    /** The String the keyCode was parsed from */
    public String keyName;

    /**
     * Get an XML representation of this Binding
     * @return XML String with the data in this binding.
     */
    public String toXML() {
       return "<binding key=\"" + keyName + "\" action=\"" + actionName + "\" />";
    }

    
}
