package cage.input;

import cage.core.Cage;
import cage.carl.Carl;
import cage.core.Logger;
import cage.core.ResourceEngine;
import cage.graphics.Sprite;
import org.minueto.handlers.MinuetoKeyboardHandler;

/**
 * Input from the keyboard is handled by an instance of the KeyboardHandler.
 * The integer values passed in here should be one of the values in Minueto
 * Keyboard. The KeyboardHandler looks through all Bindings in the program
 * for key/action bindings assocated with the given key press.
 *
 * @author Seth Marinello
 */
public class KeyboardHandler implements MinuetoKeyboardHandler {
    /**
     * Searches first the current Scene bindings(Defined in a Scene)
     * and then Global bindings (Defined in the produciton file, and if
     * a binding has been made for the pressed key, the assoicated
     * action is called.
     * @param iValue The value of the key pressed.
     */
    public void handleKeyPress(int iValue) {
        Sprite temp;

        //We first look at the binds for the current scene
        Binding action = Cage.getCurrentScene().getBindings().get(iValue);

        //If no bind was found, look in the global map
        if (action == null) {
            action = ResourceEngine.getBinding(iValue);
        }

        //Finally, if any bind was found, call the action assoicated with it.
        if (action != null) {
            Logger.getRootLogger().debug("Calling \"" + action + "\" from keypress");
            Carl.callAction(action.actionName);
        }

    }

    /**
     * Does nothing.
     * @param iValue Value of key released
     */
    public void handleKeyRelease(int iValue) {

    }

    /**
     * Does nothing.
     * @param iValue  Value of key pressed
     * @param cKeyChar Character of key pressed
     */
    public void handleKeyType(int iValue, char cKeyChar) {

    }
}
