package cage.ui;

import cage.ui.Overlay;
import cage.core.Logger;

import javax.swing.*;
import javax.swing.plaf.ScrollPaneUI;
import javax.swing.plaf.basic.BasicScrollPaneUI;
import java.awt.*;

public class GUIFactory {

    //Create and set up a colored label.
    public static Overlay.NamedPanel createColoredTextArea(String name, String text,
                                                  java.awt.Color bgColor,
                                                  java.awt.Color fgColor,
                                                  java.awt.Point origin,
                                                  java.awt.Point size,
                                                  boolean editable) {


        JTextArea jt = new JTextArea(text);
        jt.setOpaque(true);
        jt.setBackground(bgColor);
        jt.setForeground(fgColor);
        jt.setBounds(0, 0, size.x, size.y);
        jt.setEditable(editable);

        TextContainer.CageScrollText scroll = new TextContainer.CageScrollText(jt);


        scroll.setBounds(origin.x, origin.y, size.x, size.y);
        scroll.setUI((ScrollPaneUI)BasicScrollPaneUI.createUI(jt));
        scroll.setBackground(bgColor);
        scroll.setBorder(BorderFactory.createLineBorder(java.awt.Color.black));

        return wrapComponent(name, scroll);
    }

    //Create and set up a colored label.
    public static Overlay.NamedPanel createColoredLabel(String name, String text,
                                               java.awt.Color bgColor,
                                               java.awt.Color fgColor,
                                               java.awt.Point origin,
                                               int alignment) {

        TextContainer.CageLabel label = new TextContainer.CageLabel(text);
        label.setVerticalAlignment(JLabel.TOP);
        label.setHorizontalAlignment(alignment);
        label.setOpaque(true);
        label.setBackground(bgColor);
        label.setForeground(fgColor);
        label.setBorder(BorderFactory.createLineBorder(java.awt.Color.black));
        label.setBounds(origin.x, origin.y, 140, 140);
        label.setSize(140, 140);


        return wrapComponent(name, label);
    }

    private static Overlay.NamedPanel wrapComponent(String name, TextContainer container) {

        Component c = null;

        if (container instanceof Component) {
            c = (Component)container;
        }
        else {
            Logger.getRootLogger().error("Text Container is not a Component!");
            return null;
        }

        Overlay.NamedPanel guiPanel = new Overlay.NamedPanel(name);
        guiPanel.setLayout(null);

        guiPanel.setBounds((c).getBounds());
        c.setBounds(0, 0, guiPanel.getWidth(), guiPanel.getHeight());
        guiPanel.add(c);

        guiPanel.contents = container;
      
        return guiPanel;

    }
}
