package cage.ui;

import cage.ui.TextContainer;
import cage.core.ResourceEngine;

import java.awt.*;
import java.util.LinkedList;

/**
 * An overlay is simply a grouping of standard Java gui components
 * which can be added over a scene rendered in CAGE. Overlays are stored
 * as a resource an can be associated with a scene.
 *
 * @author Seth Marinello
 */
public class Overlay {

    private String name;
    private LinkedList<NamedPanel> requiredComponents;

    /**
     * Creates a new overlay with the specified name.
     * @param name name of the overlay
     */
    public Overlay(String name) {
        this.name = name;
        requiredComponents = new LinkedList<NamedPanel>();
        ResourceEngine.registerOverlay(this);
    }

    /**
     * Gets the name of the overlay
     * @return the name
     */
    public String getName() {
        return name;
    }

    /**
     * Returns all of the components to be displayed when this
     * overlay is used.
     * @return linked list of all components of the overlay
     */
    public LinkedList<NamedPanel> getComponents() {
        return requiredComponents;
    }

    /**
     * Adds a given Component to those
     * associated with the overlay.
     * @param c the component to add.
     */
    public void addComponent(NamedPanel c)  {
        requiredComponents.add(c);
    }

    /**
     * This allows access to a particular component
     * of an overly by name. This is useful for
     * setting the text displayed in the component.
     * @param name the name of the component to search for
     * @return the component named if found or null.
     */
    public NamedPanel getComponent(String name) {
       for (NamedPanel p : requiredComponents) {
          if (name.equals(p.getName())) {
             return p;
          }
       }

       return null;
    }

    public String toString() {
        return "(Overlay) " + name;
    }

    /**
     * A wrapper for AWT Panel which has a name associated
     * with it. This is used so elements of an overlay
     * can be looked up via string from CARL.
     */
    public static class NamedPanel extends Panel {
        private String name;
        public TextContainer contents;

        public NamedPanel(String name) {
            this.name = name;
        }

        public String getName() {
            return name;
        }
    }
}
