/**
 * TextContainer.java 
 *
 * Author: Seth Marinello
 * Date: Oct 3, 2007
 *
 * Copyright (c) 2006-2007 Altereddreams.
 * All Rights Reserved
 */
package cage.ui;

import javax.swing.*;

/**
 * TextContainer is an interface for GUI components which provides
 * the ability to control text displayed in component. It was created
 * because there was no way to identify Swing or AWT components which
 * have these methods via type. The internal classes CageScrollText and
 * CageLabel wrap Components which had the required methods but were not
 * identified as TextContainers.
 *
 * @author Seth Marinello
 */
public interface TextContainer  {

    /**
     * Sets the text on the Component to the specified String.
     * The exact meaning of this is up to the implementing class.
     * @param text The String to use.
     */
    public void setText(String text);

    /**
     * Provides access to the last String set on the Component
     * @return The String contents of the Component.
     */
    public String getText();

    /**
     * A ScrollPane with wrapper methods to provided access to
     * whatever JTextArea it is holding. 
     */
    public class CageScrollText extends JScrollPane implements TextContainer {

        JTextArea textArea;

        public CageScrollText(JTextArea a) {
            super(a);
            textArea = a;
        }

        public void setText(String text) {
            textArea.setText(text);
        }

        public String getText() {
            return textArea.getText();
        }
    }

    /**
     * A JLabel that has been blessed by the type gods as a TextContainer.
     * The interface is already implemented by JLabel so it is not defined here.
     */
    public class CageLabel extends JLabel implements TextContainer {

        public CageLabel(String text) {
            super(text);
        }
    }

}
