/*
 * @(#)MinuetoPanel.java 
 *
 * Minueto - The Game Development Framework 
 * Copyright (c) 2004 McGill University
 * 3480 University Street, Montreal, Quebec H3A 2A7
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.minueto.window;

import java.awt.*;

import javax.swing.JPanel;

import org.minueto.image.ImageTools;

/**
 * The <code>MinuetoPanel</code> class allows you to create a Panel which can be inserted
 * inside your Swing/AWT application. This panel is your main drawing surface (your canvas). 
 * This class is a subclass of <code>MinuetoWindow</code>. As such, all the drawing 
 * functionnality is found in the super class.
 * <p>
 * This class is EXPIREMENTAL! Don't be suprised if it doesn't work. We are working on it.
 *<p>
 * When a <code>MinuetoPanel</code> is created, it is invisible. The <code>setVisible</code>
 * method must be used toshow the window (make it visible).
 * <p>
 * Minueto draws to the screen using a double-buffering strategy. In other words, 
 * there are two drawing surfaces : your window itself and an off screen surface.
 * All your drawing occurs on the off screen surface. The content of the off 
 * screen buffer is drawn to the screen when the <code>render</code> method is invoked.
 * <p>
 *
 * Modified by Seth Marinello 2007
 *
 * @author 	Alexandre Denault
 * @version 1.0
 * @since  	Minueto 1.0
 * @see 	org.minueto.window.MinuetoWindow
 * @see   	org.minueto.image.MinuetoImage
 */
public class MinuetoPanel extends MinuetoWindow {
	
	private int width;
	private int height;
	
	private Panel jPanel;
	private Canvas canvas;
	
	public MinuetoPanel(int width, int height) {
		
		super(width, height);
				
		GraphicsDevice			graphicDevice;
		GraphicsEnvironment 	graphicEnvironment;
				
		this.visible = false;
				
		graphicEnvironment = GraphicsEnvironment.getLocalGraphicsEnvironment();
		
		/* Minueto uses the default graphic environment. We can assume this because
		 * Minueto does not yet support dual display configuration. */
		graphicDevice = graphicEnvironment.getDefaultScreenDevice();
		graphicConfiguration = graphicDevice.getDefaultConfiguration();
		
		this.jPanel = new Panel();
		this.jPanel.setSize(width, height);
        this.jPanel.setBackground(Color.BLACK);

        this.jPanel.setLayout(null);
		this.jPanel.setPreferredSize(new Dimension(width, height));
		this.jPanel.setFocusTraversalKeysEnabled(false);
		
		this.jPanel.addKeyListener(this.getKeyListener());
		this.jPanel.addMouseListener(this.getMouseListener());
		this.jPanel.addMouseMotionListener(this.getMouseMotionListener());
		
		this.canvas = new Canvas(graphicConfiguration);
		this.canvas.setIgnoreRepaint(true);
		this.canvas.setBounds( 0, 0, width, height);
		this.canvas.setFocusTraversalKeysEnabled(false);
		
		this.canvas.addKeyListener(this.getKeyListener());
		this.canvas.addMouseListener(this.getMouseListener());
		this.canvas.addMouseMotionListener(this.getMouseMotionListener());
		
		this.jPanel.add(this.canvas);
		
		ImageTools.setGraphicConfiguration(graphicConfiguration);

		WindowManager.getInstance().register(this, true);
	}
	
	public void enable() {
		
		/* We need to initialized the double-buffer mode the first time we show the window */
		if (grpGraphicsPointer == null) {
			
			/* Initialize the bufferstrategy for the frame, thus allowing us
			 * to double buffer the screen. 
			 * Note: The buffer strategy can only be created once the window
			 * is visible.*/
			this.canvas.createBufferStrategy(2);
			this.busBufferStrategy = this.canvas.getBufferStrategy();
			
			/* Create the Graphics Pointer we will need to draw on the screen.
			 * This pointer needs to be recreated everytime we render (draw) the
			 * screen. */
			this.grpGraphicsPointer = (Graphics2D)this.busBufferStrategy.getDrawGraphics();
			
			WindowManager.getInstance().register(this, false);
			
			this.visible = true;
		}
		
		
	}
	
	public void disable() {
	

	}

    /* Added by Seth Marinello when the JPanel was moved to be an AWT PANEL */
    public Panel getPanel() {
        return jPanel;
    }

    public void addToContainer(Container container) {
        container.add(this.jPanel);
	}
	
	public void removeFromContainer(Container container) {
		
		container.remove(this.jPanel);		
	}

	public void close() {
		// TODO Auto-generated method stub
		
	}

	public void setCursorVisible(boolean value) {
		// TODO Auto-generated method stub
		
	}

	public void setTitle(String title) {
		// TODO Auto-generated method stub
		
	}

	public void setVisible(boolean value) {
		
		if (value) {
			if (grpGraphicsPointer != null) {
				this.grpGraphicsPointer.dispose();
				this.grpGraphicsPointer = null;	
		
				this.getKeyListener().clear();
				this.getMouseListener().clear();
				this.getMouseMotionListener().clear();
				this.getWindowListener().clear();
				
				WindowManager.getInstance().unregister(this);
				
				this.visible = false;
			}
		}		
	}
	

}
